import { PageWrapper } from '@/components/foundation'
import { auth, currentUser } from '@clerk/nextjs/server'
import { notFound } from 'next/navigation'
import Link from 'next/link'
import { Octokit } from '@octokit/rest'
import { Shapes } from 'lucide-react'
import { Button } from '@workspace/ui/components/button'

/**
 * Dashboard page
 * @returns {React.ReactNode} The rendered component
 */
export default async function Page() {
  const authenticated = await auth()
  const userId = authenticated.userId

  if (!userId) {
    return notFound()
  }

  try {
    const user = await currentUser()
    const githubAccount = user?.externalAccounts.find(
      (account) => account.provider === 'oauth_github'
    )

    if (!githubAccount) {
      return (
        <PageWrapper 
          header={{ 
            title: 'Dashboard',
            actions: [{ label: 'Create Project', href: '/projects/create' }]
          }}
          layout="bento"
          className="pb-0"
        >
          <div className="md:col-span-3 border border-gray-800 rounded-lg min-h-[600px] flex flex-col items-center justify-center p-6">
            <div className="mb-6">
              <div className="flex flex-col items-center">
                <Shapes size={64} className="stroke-current" />
              </div>
            </div>
            <div className="text-2xl font-bold mb-2">
              GitHub Account Not Connected
            </div>
            <div className="text-muted-foreground text-center max-w-md mb-6">
              You need to connect your GitHub account to use the dashboard features. 
              Please visit your user profile in Clerk to connect GitHub.
            </div>
            <Button 
              className="bg-white text-black hover:bg-gray-200 flex items-center"
              asChild
            >
              <Link href="https://accounts.clerk.dev/user" target="_blank">
                <svg className="mr-2 h-4 w-4" viewBox="0 0 24 24" fill="currentColor" xmlns="http://www.w3.org/2000/svg">
                  <path d="M12 0C5.37 0 0 5.37 0 12c0 5.31 3.435 9.795 8.205 11.385.6.105.825-.255.825-.57 0-.285-.015-1.23-.015-2.235-3.015.555-3.795-.735-4.035-1.41-.135-.345-.72-1.41-1.23-1.695-.42-.225-1.02-.78-.015-.795.945-.015 1.62.87 1.845 1.23 1.08 1.815 2.805 1.305 3.495.99.105-.78.42-1.305.765-1.605-2.67-.3-5.46-1.335-5.46-5.925 0-1.305.465-2.385 1.23-3.225-.12-.3-.54-1.53.12-3.18 0 0 1.005-.315 3.3 1.23.96-.27 1.98-.405 3-.405s2.04.135 3 .405c2.295-1.56 3.3-1.23 3.3-1.23.66 1.65.24 2.88.12 3.18.765.84 1.23 1.905 1.23 3.225 0 4.605-2.805 5.625-5.475 5.925.435.375.81 1.095.81 2.22 0 1.605-.015 2.895-.015 3.3 0 .315.225.69.825.57A12.02 12.02 0 0024 12c0-6.63-5.37-12-12-12z" />
                </svg>
                Connect to GitHub
              </Link>
            </Button>
          </div>
        </PageWrapper>
      )
    }
    
    // NOTE: We're keeping the token approach for now, but aware it's not working
    const authToken = githubAccount.accessToken;
    
    // Try using GitHub token
    let octokit;
    try {
      octokit = new Octokit({
        auth: authToken || process.env.GITHUB_TOKEN
      });
      
      // Test with a simple request
      
      // Try listing repositories
      const repoResponse = await octokit.repos.listForAuthenticatedUser();
      
      return (
        <PageWrapper 
          header={{ 
            title: 'Dashboard',
            actions: [{ label: 'Create Project', href: '/projects/create' }]
          }}
          layout="bento"
          className="pb-0"
        >
          <div className="md:col-span-3">
            <div className="grid grid-cols-1 sm:grid-cols-2 lg:grid-cols-3 gap-4">
              {repoResponse.data.length > 0 ? (
                repoResponse.data.map((repo) => (
                  <div 
                    key={repo.id} 
                    className="border border-gray-800 rounded-lg p-6 flex flex-col"
                  >
                    <h3 className="text-lg font-medium mb-2">{repo.name}</h3>
                    <p className="text-muted-foreground text-sm mb-4 flex-grow">
                      {repo.description || 'No description provided'}
                    </p>
                    <div className="flex items-center justify-between">
                      <div className="flex items-center">
                        <span className="text-sm text-muted-foreground">
                          {repo.default_branch}
                        </span>
                      </div>
                      <Link 
                        href={repo.html_url} 
                        target="_blank" 
                        className="text-sm text-blue-400 hover:underline"
                      >
                        View on GitHub
                      </Link>
                    </div>
                  </div>
                ))
              ) : (
                <div className="md:col-span-3 flex justify-center items-center min-h-[400px]">
                  <p className="text-muted-foreground">No repositories found</p>
                </div>
              )}
            </div>
          </div>
        </PageWrapper>
      );
    } catch (authError) {
      console.error("GitHub API error:", authError);
      return (
        <PageWrapper 
          header={{ 
            title: 'Dashboard',
            actions: [{ label: 'Create Project', href: '/projects/create' }]
          }}
          layout="bento"
          className="pb-0"
        >
          <div className="md:col-span-3 border border-gray-800 rounded-lg min-h-[600px] flex flex-col items-center justify-center p-6">
            <div className="mb-6">
              <div className="flex flex-col items-center">
                <Shapes size={64} className="stroke-current" />
              </div>
            </div>
            <div className="text-xl font-bold mb-2">
              Failed to access GitHub API
            </div>
            <div className="text-red-500 mb-4">
              {authError.message}
            </div>
            <div className="text-muted-foreground text-center max-w-md mb-6">
              <p>This issue may be related to how Clerk is managing the GitHub token.</p>
              <p className="mt-2">Try reconnecting your GitHub account with the correct permissions.</p>
            </div>
            <Button 
              className="bg-white text-black hover:bg-gray-200 flex items-center"
              asChild
            >
              <Link href="https://accounts.clerk.dev/user" target="_blank">
                <svg className="mr-2 h-4 w-4" viewBox="0 0 24 24" fill="currentColor" xmlns="http://www.w3.org/2000/svg">
                  <path d="M12 0C5.37 0 0 5.37 0 12c0 5.31 3.435 9.795 8.205 11.385.6.105.825-.255.825-.57 0-.285-.015-1.23-.015-2.235-3.015.555-3.795-.735-4.035-1.41-.135-.345-.72-1.41-1.23-1.695-.42-.225-1.02-.78-.015-.795.945-.015 1.62.87 1.845 1.23 1.08 1.815 2.805 1.305 3.495.99.105-.78.42-1.305.765-1.605-2.67-.3-5.46-1.335-5.46-5.925 0-1.305.465-2.385 1.23-3.225-.12-.3-.54-1.53.12-3.18 0 0 1.005-.315 3.3 1.23.96-.27 1.98-.405 3-.405s2.04.135 3 .405c2.295-1.56 3.3-1.23 3.3-1.23.66 1.65.24 2.88.12 3.18.765.84 1.23 1.905 1.23 3.225 0 4.605-2.805 5.625-5.475 5.925.435.375.81 1.095.81 2.22 0 1.605-.015 2.895-.015 3.3 0 .315.225.69.825.57A12.02 12.02 0 0024 12c0-6.63-5.37-12-12-12z" />
                </svg>
                Connect to GitHub
              </Link>
            </Button>
          </div>
        </PageWrapper>
      );
    }
  } catch (error) {
    console.error("GitHub authentication error:", error);
    return (
      <PageWrapper 
        header={{ 
          title: 'Dashboard',
          actions: [{ label: 'Create Project', href: '/projects/create' }]
        }}
        layout="bento"
        className="pb-0"
      >
        <div className="md:col-span-3 flex flex-col items-center justify-center min-h-[600px]">
          <div className="text-2xl font-bold mb-4">
            Failed to authenticate with GitHub
          </div>
          <div className="text-red-500">
            {error.message}
          </div>
        </div>
      </PageWrapper>
    );
  }
}