'use client'
import { PageWrapper } from '@/components/foundation'
import CheckBalanceIframe from '@/components/iframe/check-balance-iframe/CheckBalanceIframe'
import { FixedProjectCard } from '@/components/projects/project/ProjectCard/FixedProjectCard'
import { Button } from '@workspace/ui/components/button'
import { useEffect, useState } from 'react'
import { Shapes } from 'lucide-react'
import { useGQLClient } from '@/context'
import type { Project } from '@workspace/gql-client'

interface ProjectData {
  id: string
  name: string
  icon?: string
  deployments: any[]
  repository?: string
  framework?: string
  description?: string
}

export default function ProjectsPage() {
  const [, setIsBalanceSufficient] = useState<boolean>()
  const [projects, setProjects] = useState<ProjectData[]>([])
  const [isLoading, setIsLoading] = useState<boolean>(true)
  const [error, setError] = useState<string | null>(null)
  
  const client = useGQLClient()

  const handleCreateProject = () => {
    window.location.href = '/projects/github/ps/cr'
  }

  useEffect(() => {
    loadAllProjects()
  }, [])

  const loadAllProjects = async () => {
    try {
      setIsLoading(true)
      setError(null)
      
      // First get organizations
      const orgsResponse = await client.getOrganizations()
      
      if (!orgsResponse.organizations || orgsResponse.organizations.length === 0) {
        setProjects([])
        setIsLoading(false)
        return
      }
      
      // Get projects from all organizations
      const allProjects: ProjectData[] = []
      
      for (const org of orgsResponse.organizations) {
        try {
          const projectsResponse = await client.getProjectsInOrganization(org.slug)
          
          // Transform GraphQL projects to match ProjectData interface
          const orgProjects: ProjectData[] = projectsResponse.projectsInOrganization.map((project: Project) => ({
            id: project.id,
            name: project.name,
            repository: project.repository,
            framework: project.framework,
            description: project.description,
            deployments: project.deployments || []
          }))
          
          allProjects.push(...orgProjects)
        } catch (orgError) {
          console.error(`Failed to load projects for org ${org.slug}:`, orgError)
          // Continue with other orgs even if one fails
        }
      }
      
      setProjects(allProjects)
    } catch (err) {
      console.error('Failed to load projects:', err)
      setError(err instanceof Error ? err.message : 'Failed to load projects')
    } finally {
      setIsLoading(false)
    }
  }
  
  return (
    <PageWrapper
      header={{
        title: 'Projects',
        actions: [{ label: 'Create Project', href: '/projects/github/ps/cr' }]
      }}
      layout="bento"
      className="pb-0"
    >
      {isLoading ? (
        // Full width loading spinner in bento layout
        <div className="md:col-span-3 flex justify-center items-center min-h-[600px]">
          <div className="animate-spin rounded-full h-12 w-12 border-t-2 border-b-2 border-primary"></div>
        </div>
      ) : error ? (
        // Full width error state in bento layout
        <div className="md:col-span-3 border border-gray-800 rounded-lg min-h-[600px] flex flex-col items-center justify-center p-6">
          <div className="mb-6">
            <div className="flex flex-col items-center">
              <Shapes size={64} className="stroke-current" />
            </div>
          </div>
          <h2 className="text-xl font-semibold mb-2">Error: {error}</h2>
          <p className="text-gray-400 text-center max-w-md mb-6">
            Failed to load your deployed projects. Please try again.
          </p>
          <Button 
            className="bg-white text-black hover:bg-gray-200 flex items-center"
            onClick={loadAllProjects}
          >
            Try Again
          </Button>
        </div>
      ) : projects.length === 0 ? (
        // Full width empty state in bento layout
        <div className="md:col-span-3 border border-gray-800 rounded-lg min-h-[600px] flex flex-col items-center justify-center p-6">
          <div className="mb-6">
            <div className="flex flex-col items-center">
              <Shapes size={64} className="stroke-current" />
            </div>
          </div>
          <h2 className="text-xl font-semibold mb-2">Deploy your first app</h2>
          <p className="text-gray-400 text-center max-w-md mb-6">
            You don't have any deployed projects yet. Create your first project to get started.
          </p>
          <Button 
            className="bg-white text-black hover:bg-gray-200 flex items-center"
            onClick={handleCreateProject}
          >
            <svg className="mr-2 h-4 w-4" viewBox="0 0 24 24" fill="none" stroke="currentColor" strokeWidth="2">
              <line x1="12" y1="5" x2="12" y2="19"></line>
              <line x1="5" y1="12" x2="19" y2="12"></line>
            </svg>
            Create Project
          </Button>
        </div>
      ) : (
        // Custom grid that spans the entire bento layout
        <div className="md:col-span-3">
          <div className="grid grid-cols-1 sm:grid-cols-2 lg:grid-cols-3 gap-4">
            {projects.map((project) => {
              // Get the current deployment for status
              const currentDeployment = project.deployments.find(d => d.isCurrent)
              const latestDeployment = project.deployments[0] // Assuming sorted by date
              
              // Determine status based on deployment
              let status = 'pending'
              if (currentDeployment || latestDeployment) {
                const deployment = currentDeployment || latestDeployment
                switch (deployment.status?.toUpperCase()) {
                  case 'READY':
                  case 'COMPLETED':
                    status = 'success'
                    break
                  case 'BUILDING':
                  case 'DEPLOYING':
                    status = 'in-progress'
                    break
                  case 'ERROR':
                  case 'FAILED':
                    status = 'failure'
                    break
                  default:
                    status = 'pending'
                }
              }
              
              // Format the project data to match what FixedProjectCard expects
              const formattedProject = {
                id: project.id,
                name: project.name,
                full_name: project.repository ? project.repository.replace('https://github.com/', '') : project.name,
                repository: project.repository,
                framework: project.framework,
                description: project.description,
                // Ensure deployments array is properly formatted
                deployments: project.deployments.map(deployment => ({
                  ...deployment,
                  // Make sure the date is in a format the card can parse
                  createdAt: deployment.createdAt,
                  applicationDeploymentRecordData: {
                    url: deployment.applicationDeploymentRecordData?.url || `https://${project.name.toLowerCase()}.example.com`
                  }
                }))
              }
              
              return (
                <FixedProjectCard 
                  project={formattedProject} 
                  key={project.id} 
                  status={status as any}
                />
              )
            })}
          </div>
        </div>
      )}
      
      {/* Wrap in try/catch to prevent breaking if there are issues */}
      {(() => {
        try {
          return (
            <CheckBalanceIframe
              onBalanceChange={setIsBalanceSufficient}
              isPollingEnabled={false}
              amount="1"
            />
          );
        } catch (error) {
          console.error('Failed to render CheckBalanceIframe:', error);
          return null;
        }
      })()}
    </PageWrapper>
  )
}