'use client'

import type React from 'react'

import {
  Bell,
  ChevronRight,
  CreditCard,
  Globe,
  HelpCircle,
  Lock,
  Shield,
  User
} from 'lucide-react'
import Link from 'next/link'
import { useState } from 'react'

import { ComingSoonOverlay } from '@/components/foundation'
import { Button } from '@workspace/ui/components/button'
import { Separator } from '@workspace/ui/components/separator'
import { useRouter } from 'next/navigation'
/**
 * Settings category item component
 * Renders a single settings category with an icon and label
 */
interface SettingsCategoryProps {
  /** The icon to display for this category */
  icon: React.ReactNode
  /** The label text for this category */
  label: string
  /** Whether this category is currently active */
  isActive?: boolean
}

function SettingsCategory({
  icon,
  label,
  isActive = false
}: SettingsCategoryProps) {
  return (
    <Link
      href="#"
      className={`flex items-center gap-3 px-4 py-2 rounded-md transition-colors ${
        isActive ? 'bg-muted font-medium' : 'hover:bg-muted/50'
      }`}
    >
      {icon}
      <span>{label}</span>
      {isActive && <ChevronRight className="ml-auto h-4 w-4" />}
    </Link>
  )
}

/**
 * Settings page component
 * Displays a simple settings interface with categories and placeholder content
 */
export default function SettingsPage() {
  const router = useRouter()
  const [activeCategory] = useState('profile')
  const [formState, setFormState] = useState({
    darkTheme: false,
    autoDetectTimezone: true,
    allowAnalytics: true
  })

  /**
   * Handle checkbox changes
   */
  const handleCheckboxChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    const { id, checked } = e.target
    setFormState((prev) => ({
      ...prev,
      [id]: checked
    }))
  }

  return (
    <div className="min-h-screen bg-background">
      <ComingSoonOverlay
        message="Settings configuration will be available in the next release."
        routerAction={() => router.back()}
      />

      {/* Header */}
      <header className="border-b">
        <div className="container py-4">
          <div className="flex items-center">
            <Link href="/" className="flex items-center gap-2">
              <Globe className="h-6 w-6" />
              <span className="font-bold">Laconic Deploy</span>
            </Link>
            <h1 className="text-xl font-semibold ml-8">Settings</h1>
          </div>
        </div>
      </header>

      <div className="container py-8">
        <div className="grid grid-cols-1 md:grid-cols-[250px_1fr] gap-8">
          {/* Sidebar */}
          <nav className="space-y-1">
            <SettingsCategory
              icon={<User className="h-5 w-5" />}
              label="Profile"
              isActive={activeCategory === 'profile'}
            />
            <SettingsCategory
              icon={<Bell className="h-5 w-5" />}
              label="Notifications"
            />
            <SettingsCategory
              icon={<Lock className="h-5 w-5" />}
              label="Password & Security"
            />
            <SettingsCategory
              icon={<Shield className="h-5 w-5" />}
              label="Access Control"
            />
            <SettingsCategory
              icon={<CreditCard className="h-5 w-5" />}
              label="Billing"
            />
            <SettingsCategory
              icon={<HelpCircle className="h-5 w-5" />}
              label="Help & Support"
            />
          </nav>

          {/* Main content */}
          <div className="space-y-6">
            <div>
              <h2 className="text-2xl font-bold tracking-tight">
                Profile Settings
              </h2>
              <p className="text-muted-foreground">
                Manage your account information and preferences.
              </p>
            </div>

            <Separator />

            <div className="space-y-4">
              <div className="grid gap-2">
                <h3 className="text-lg font-medium">Personal Information</h3>
                <p className="text-sm text-muted-foreground">
                  Update your personal details and how we contact you.
                </p>
              </div>

              <div className="grid gap-6 md:grid-cols-2">
                <div className="space-y-2">
                  <label htmlFor="name" className="text-sm font-medium">
                    Full Name
                  </label>
                  <input
                    id="name"
                    type="text"
                    placeholder="John Doe"
                    className="w-full px-3 py-2 border rounded-md"
                  />
                </div>

                <div className="space-y-2">
                  <label htmlFor="email" className="text-sm font-medium">
                    Email Address
                  </label>
                  <input
                    id="email"
                    type="email"
                    placeholder="john@example.com"
                    className="w-full px-3 py-2 border rounded-md"
                  />
                </div>
              </div>

              <div className="flex justify-end">
                <Button>Save Changes</Button>
              </div>
            </div>

            <Separator />

            <div className="space-y-4">
              <div className="grid gap-2">
                <h3 className="text-lg font-medium">Preferences</h3>
                <p className="text-sm text-muted-foreground">
                  Customize your experience with Laconic Deploy.
                </p>
              </div>

              <div className="space-y-4">
                <div className="flex items-center space-x-2">
                  <input
                    id="darkTheme"
                    type="checkbox"
                    className="rounded"
                    checked={formState.darkTheme}
                    onChange={handleCheckboxChange}
                  />
                  <label htmlFor="darkTheme" className="text-sm font-medium">
                    Use dark theme
                  </label>
                </div>

                <div className="flex items-center space-x-2">
                  <input
                    id="autoDetectTimezone"
                    type="checkbox"
                    className="rounded"
                    checked={formState.autoDetectTimezone}
                    onChange={handleCheckboxChange}
                  />
                  <label
                    htmlFor="autoDetectTimezone"
                    className="text-sm font-medium"
                  >
                    Automatically detect timezone
                  </label>
                </div>

                <div className="flex items-center space-x-2">
                  <input
                    id="allowAnalytics"
                    type="checkbox"
                    className="rounded"
                    checked={formState.allowAnalytics}
                    onChange={handleCheckboxChange}
                  />
                  <label
                    htmlFor="allowAnalytics"
                    className="text-sm font-medium"
                  >
                    Allow anonymous usage data collection
                  </label>
                </div>
              </div>

              <div className="flex justify-end">
                <Button>Save Preferences</Button>
              </div>
            </div>
          </div>
        </div>
      </div>

      <footer className="border-t mt-auto">
        <div className="container py-6">
          <p className="text-center text-sm text-muted-foreground">
            © {new Date().getFullYear()} Laconic Deploy. All rights reserved.
          </p>
        </div>
      </footer>
    </div>
  )
}
