import { createHmac, timingSafeEqual } from 'node:crypto'
import type { WebhookEventName } from '@octokit/webhooks-types'

// Add GET handler for GitHub webhook verification

export async function GET() {
  return new Response('Ready to receive webhooks', { status: 200 })
}

export async function POST(request: Request) {
  try {
    const payload = await request.text()
    const signature256 = request.headers.get('x-hub-signature-256')
    const signature1 = request.headers.get('x-hub-signature')
    const eventType = request.headers.get('x-github-event')
    const event = eventType as WebhookEventName

    console.log('Received webhook:', {
      event,
      signatures: { signature256, signature1 }
    })

    // Always use SHA256 if available
    if (!signature256) {
      return new Response('SHA256 signature required', { status: 401 })
    }

    const secret = process.env.GITHUB_WEBHOOK_SECRET
    if (!secret) {
      return new Response('Webhook secret not configured', { status: 500 })
    }

    // Calculate expected SHA256 hash
    const expectedHash = createHmac('sha256', secret)
      .update(payload)
      .digest('hex')

    const providedHash = signature256.replace('sha256=', '')

    console.log('Verifying signature:', {
      provided: providedHash,
      expected: expectedHash,
      match: providedHash === expectedHash
    })

    const signatureMatches = timingSafeEqual(
      Buffer.from(providedHash),
      Buffer.from(expectedHash)
    )

    if (!signatureMatches) {
      return new Response('Invalid signature', { status: 401 })
    }

    const data = JSON.parse(payload)
    console.log('Processing webhook:', { event, data })

    return new Response('OK', { status: 200 })
  } catch (error: unknown) {
    console.error('Webhook error:', error)
    return new Response(
      error instanceof Error ? error.message : 'Unknown error',
      { status: 500 }
    )
  }
}
