// src/app/auth/github/backend-callback/page.tsx
'use client'
import { useEffect, useState } from 'react'
import { useSearchParams } from 'next/navigation'
import { Card, CardContent, CardDescription, CardHeader, CardTitle } from '@workspace/ui/components/card'
import { CheckCircle2, AlertCircle, Loader2 } from 'lucide-react'
import { useGQLClient } from '@/context'

export default function GitHubBackendCallbackPage() {
  const searchParams = useSearchParams()
  const [status, setStatus] = useState<'processing' | 'success' | 'error'>('processing')
  const [message, setMessage] = useState('Processing GitHub authentication...')
  const gqlClient = useGQLClient()
  
  useEffect(() => {
    const handleCallback = async () => {
      try {
        // Get parameters from URL
        const code = searchParams.get('code')
        const state = searchParams.get('state')
        const error = searchParams.get('error')
        const errorDescription = searchParams.get('error_description')
        
        // Check for OAuth errors
        if (error) {
          throw new Error(errorDescription || `GitHub OAuth error: ${error}`)
        }
        
        // Validate required parameters
        if (!code) {
          throw new Error('No authorization code received from GitHub')
        }
        
        // Verify state parameter for security
        const storedState = sessionStorage.getItem('github_oauth_state')
        if (state !== storedState) {
          throw new Error('Invalid state parameter - possible CSRF attack')
        }
        
        // Clean up stored state
        sessionStorage.removeItem('github_oauth_state')
        
        setMessage('Connecting to backend...')
        
        // Call backend's authenticateGitHub mutation
        const result = await gqlClient.authenticateGitHub(code)
        
        if (result.authenticateGitHub?.token) {
          setStatus('success')
          setMessage('GitHub authentication successful!')
          
          // Notify parent window
          if (window.opener) {
            window.opener.postMessage({
              type: 'GITHUB_BACKEND_AUTH_SUCCESS',
              token: result.authenticateGitHub.token
            }, window.location.origin)
          }
          
          // Close popup after a short delay
          setTimeout(() => {
            window.close()
          }, 2000)
        } else {
          throw new Error('No token received from backend')
        }
        
      } catch (error) {
        console.error('GitHub OAuth callback error:', error)
        setStatus('error')
        setMessage(error instanceof Error ? error.message : 'Unknown error occurred')
        
        // Notify parent window of error
        if (window.opener) {
          window.opener.postMessage({
            type: 'GITHUB_BACKEND_AUTH_ERROR',
            message: error instanceof Error ? error.message : 'Unknown error'
          }, window.location.origin)
        }
        
        // Close popup after delay even on error
        setTimeout(() => {
          window.close()
        }, 5000)
      }
    }
    
    // Only run if we have search params (meaning this is the callback)
    if (searchParams.toString()) {
      handleCallback()
    }
  }, [searchParams, gqlClient])
  
  const getStatusIcon = () => {
    switch (status) {
      case 'processing':
        return <Loader2 className="h-8 w-8 animate-spin text-blue-600" />
      case 'success':
        return <CheckCircle2 className="h-8 w-8 text-green-600" />
      case 'error':
        return <AlertCircle className="h-8 w-8 text-red-600" />
    }
  }
  
  const getStatusColor = () => {
    switch (status) {
      case 'processing':
        return 'text-blue-800'
      case 'success':
        return 'text-green-800'
      case 'error':
        return 'text-red-800'
    }
  }
  
  return (
    <div className="min-h-screen flex items-center justify-center bg-gray-50 dark:bg-gray-900 p-4">
      <Card className="w-full max-w-md">
        <CardHeader className="text-center">
          <div className="flex justify-center mb-4">
            {getStatusIcon()}
          </div>
          <CardTitle>GitHub Authentication</CardTitle>
          <CardDescription>
            {status === 'processing' && 'Processing your GitHub authentication...'}
            {status === 'success' && 'Authentication completed successfully'}
            {status === 'error' && 'Authentication failed'}
          </CardDescription>
        </CardHeader>
        <CardContent>
          <div className="text-center space-y-4">
            <p className={`text-sm ${getStatusColor()}`}>
              {message}
            </p>
            
            {status === 'success' && (
              <div className="text-xs text-gray-500">
                This window will close automatically...
              </div>
            )}
            
            {status === 'error' && (
              <div className="space-y-2">
                <div className="text-xs text-gray-500">
                  This window will close automatically in a few seconds.
                </div>
                <button 
                  onClick={() => window.close()}
                  className="text-sm text-blue-600 hover:text-blue-800 underline"
                >
                  Close manually
                </button>
              </div>
            )}
            
            {status === 'processing' && (
              <div className="text-xs text-gray-500">
                Please wait while we complete the authentication process...
              </div>
            )}
          </div>
        </CardContent>
      </Card>
    </div>
  )
}