// src/app/test-connection/page.tsx
'use client'
import { useState, useEffect } from 'react'
import { PageWrapper } from '@/components/foundation'
import { DirectKeyAuth } from '@/components/DirectKeyAuth'
import { GQLTest } from '@/components/GQLTest'
import { Tabs, TabsContent, TabsList, TabsTrigger } from '@workspace/ui/components/tabs'
import { Card, CardContent, CardDescription, CardHeader, CardTitle } from '@workspace/ui/components/card'
import { useGQLClient } from '@/context'
import { Button } from '@workspace/ui/components/button'
import { Input } from '@workspace/ui/components/input'
import { Label } from '@workspace/ui/components/label'
import { Loader2, AlertTriangle, CheckCircle2, GitBranch } from 'lucide-react'
import { toast } from 'sonner'
import { useRepoData } from '@/hooks/useRepoData'
import { useAuth, useUser, SignIn } from "@clerk/nextjs"
import { GitHubBackendAuth } from '@/components/GitHubBackendAuth'

// Add this at the very top of your file
declare global {
  interface Window {
    Clerk?: {
      session?: {
        getToken: (options?: { template?: string }) => Promise<string | null>
      }
    }
  }
}

export default function TestConnectionPage() {
  // Get getToken from useAuth hook, not from user
  const { isSignedIn, isLoaded: isClerkLoaded, getToken } = useAuth()
  const { user, isLoaded: isUserLoaded } = useUser()

  // Authentication states
  const [isWalletConnected, setIsWalletConnected] = useState(false)
  const [isBackendConnected, setIsBackendConnected] = useState(false)
  const [isGithubAuthed, setIsGithubAuthed] = useState(false)
  
  // Organization and deployment states
  const [organizations, setOrganizations] = useState<any[]>([])
  const [selectedOrg, setSelectedOrg] = useState<string>('')
  const [isDeploying, setIsDeploying] = useState(false)
  const [deploymentResult, setDeploymentResult] = useState<any>(null)
  const [deploymentError, setDeploymentError] = useState<string | null>(null)
  
  const [deployers, setDeployers] = useState<any[]>([])
  const [selectedDeployer, setSelectedDeployer] = useState<string>('')
  const [deployersLoading, setDeployersLoading] = useState(false)

  // Form state
  const [formData, setFormData] = useState({
    name: 'test-deployment',
    repository: '',
    branch: 'main',
  })
  
  // Contexts and hooks
  const gqlClient = useGQLClient()
  
  // Use the useRepoData hook to get repositories (using Clerk's GitHub integration)
  const { repoData: repositories } = useRepoData('')
  
  // Check if both authentications are complete
  const isFullyAuthenticated = isWalletConnected && isBackendConnected && isSignedIn && isGithubAuthed
  // Add this near your other useState declarations at the top of the component
  const [manualToken, setManualToken] = useState('')

  // Update the function to use getToken from useAuth
  const getClerkTokenForManualEntry = async () => {
    if (!isSignedIn || !user) {
      toast.error('Please sign in first')
      return
    }
    
    try {
      console.log('Attempting to get token from useAuth...')
      
      // Method 1: Try getToken from useAuth hook
      let token = null
      
      try {
        token = await getToken()
        console.log('Method 1 (getToken from useAuth) worked:', token ? 'SUCCESS' : 'NO TOKEN')
      } catch (error) {
        console.log('Method 1 failed:', error)
      }
      
      // Method 2: Try with template parameter
      if (!token) {
        try {
          token = await getToken({ template: 'github' })
          console.log('Method 2 (getToken with github template) worked:', token ? 'SUCCESS' : 'NO TOKEN')
        } catch (error) {
          console.log('Method 2 failed:', error)
        }
      }
      
      // Method 3: Try accessing window.Clerk (as mentioned in discussions)
      if (!token && typeof window !== 'undefined' && window.Clerk) {
        try {
          token = await window.Clerk.session?.getToken()
          console.log('Method 3 (window.Clerk.session.getToken) worked:', token ? 'SUCCESS' : 'NO TOKEN')
        } catch (error) {
          console.log('Method 3 failed:', error)
        }
      }
      
      // Method 4: Try window.Clerk with template
      if (!token && typeof window !== 'undefined' && window.Clerk) {
        try {
          token = await window.Clerk.session?.getToken({ template: 'github' })
          console.log('Method 4 (window.Clerk with github template) worked:', token ? 'SUCCESS' : 'NO TOKEN')
        } catch (error) {
          console.log('Method 4 failed:', error)
        }
      }
      
      if (token) {
        setManualToken(token)
        // Copy to clipboard automatically
        navigator.clipboard.writeText(token)
        toast.success('Token extracted and copied to clipboard')
        console.log('GitHub token from Clerk:', token.substring(0, 20) + '...')
      } else {
        toast.error('Unable to extract GitHub token. Check console for details.')
        console.log('GitHub account object:', user.externalAccounts?.find(account => account.provider === 'github'))
      }
    } catch (error) {
      console.error('Error getting token from Clerk:', error)
      toast.error(`Failed to get token: ${error instanceof Error ? error.message : 'Unknown error'}`)
    }
  }

  // Check backend connection
  const checkBackendConnection = async () => {
    try {
      // Test session
      const response = await fetch('http://localhost:8000/auth/session', {
        method: 'GET',
        credentials: 'include',
      })
      
      if (response.ok) {
        setIsBackendConnected(true)
        console.log('Backend connected!')
        
        // Check if user has GitHub token in backend
        await checkBackendGithubAuth()
        
        // Fetch organizations
        try {
          const orgsData = await gqlClient.getOrganizations()
          console.log('Organizations:', orgsData)
          setOrganizations(orgsData.organizations || [])
          
          // Set default org if available
          if (orgsData.organizations && orgsData.organizations.length > 0) {
            setSelectedOrg(orgsData.organizations[0].slug)
          }
        } catch (error) {
          console.error('Error fetching organizations:', error)
        }
      } else {
        setIsBackendConnected(false)
        console.log('Backend not connected')
      }
    } catch (error) {
      console.error('Error checking backend connection:', error)
      setIsBackendConnected(false)
    }
  }
  
  // Check if user has GitHub token in the backend database
  const checkBackendGithubAuth = async () => {
    try {
      // Try to get user data from backend
      const userData = await gqlClient.getUser()
      console.log('Backend user data:', userData)
      
      // Check if user has GitHub token in backend
      setIsGithubAuthed(!!userData.user.gitHubToken)
    } catch (error) {
      console.error('Error checking backend GitHub auth:', error)
      setIsGithubAuthed(false)
    }
  }
  
  // Sync GitHub token from Clerk to backend
  // Check wallet connection status whenever the backend connection changes
  useEffect(() => {
    const checkWalletConnection = async () => {
      if (isBackendConnected) {
        try {
          const response = await fetch('http://localhost:8000/auth/session', {
            method: 'GET',
            credentials: 'include',
          })
          
          setIsWalletConnected(response.ok)
        } catch (error) {
          console.error('Error checking wallet connection:', error)
          setIsWalletConnected(false)
        }
      }
    }
    
    checkWalletConnection()
  }, [isBackendConnected])
  
  // Check backend connection on mount
  useEffect(() => {
    if (isClerkLoaded && isUserLoaded) {
      checkBackendConnection()
    }
  }, [isClerkLoaded, isUserLoaded])
  
  const handleChange = (e: React.ChangeEvent<HTMLInputElement | HTMLSelectElement>) => {
    const { name, value } = e.target
    setFormData(prev => ({ ...prev, [name]: value }))
  }
  
  // Add this function to fetch deployers:
const fetchDeployers = async () => {
  try {
    setDeployersLoading(true)
    const deployersData = await gqlClient.getDeployers()
    console.log('Available deployers:', deployersData)
    setDeployers(deployersData.deployers || [])
    
    // Auto-select first deployer if available
    if (deployersData.deployers && deployersData.deployers.length > 0) {
      setSelectedDeployer(deployersData.deployers[0].deployerLrn)
    }
  } catch (error) {
    console.error('Error fetching deployers:', error)
    toast.error('Failed to fetch deployers')
  } finally {
    setDeployersLoading(false)
  }
}

// Add this useEffect to fetch deployers when backend is connected:
useEffect(() => {
  if (isBackendConnected && isFullyAuthenticated) {
    fetchDeployers()
  }
}, [isBackendConnected, isFullyAuthenticated])

// Updated handleDeploy function:
const handleDeploy = async () => {
  if (!isFullyAuthenticated) {
    setDeploymentError('Complete authentication required. Please authenticate with both wallet and GitHub.')
    return
  }
  
  if (!selectedOrg) {
    setDeploymentError('No organization selected')
    return
  }
  
  if (!formData.repository) {
    setDeploymentError('No repository selected')
    return
  }
  
  if (!selectedDeployer) {
    setDeploymentError('No deployer selected')
    return
  }
  
  setIsDeploying(true)
  setDeploymentError(null)
  setDeploymentResult(null)
  
  try {
    console.log('🚀 Starting deployment with data:', {
      ...formData,
      organizationSlug: selectedOrg,
      deployerLrn: selectedDeployer
    })
    
    // Validate repository format
    if (!formData.repository.includes('/')) {
      throw new Error('Repository must be in format "owner/repo-name"')
    }
    
    const [owner, repo] = formData.repository.split('/')
    if (!owner || !repo) {
      throw new Error('Invalid repository format. Expected "owner/repo-name"')
    }
    
    console.log('📤 Calling backend addProject mutation...')
    
    // Use the addProject mutation with deployer LRN
    const result = await gqlClient.addProject(
      selectedOrg,
      {
        name: formData.name,
        repository: formData.repository,
        prodBranch: formData.branch,
        paymentAddress: "0x1ac42F4A25Ae0137d10a825a2e33e32de0F6B57E",
        txHash: "0x0000000000000000000000000000000000000000000000000000000000000000"
      },
      selectedDeployer, // Pass the deployer LRN here
      undefined, // auctionParams
      [] // environmentVariables
    )
    
    console.log('Project creation result:', result)
    
    if (result.addProject?.id) {
      // Wait a moment to allow deployment to start
      await new Promise(resolve => setTimeout(resolve, 2000))
      
      // Get updated project data with deployments
      const projectData = await gqlClient.getProject(result.addProject.id)
      console.log('Project data with deployments:', projectData)
      
      setDeploymentResult({
        project: projectData.project,
        message: 'Project created successfully!'
      })
      
      toast.success('Project deployed successfully!')
    } else {
      throw new Error('No project ID returned from creation')
    }
  } catch (error) {
    console.error('Deployment failed:', error)
    
    let errorMessage = 'Unknown error'
    if (error instanceof Error) {
      errorMessage = error.message
    }
    
    setDeploymentError(`Failed to deploy: ${errorMessage}`)
    toast.error('Deployment failed')
  } finally {
    setIsDeploying(false)
  }
}

  return (
    <PageWrapper
      header={{
        title: 'Connection & Deployment Test',
        description: 'Test backend connection, authentication, and deployment functionality'
      }}
    >
      <div className="grid grid-cols-1 gap-8 mt-4">
        <Card>
          <CardHeader>
            <CardTitle className="flex items-center">
              Authentication Status
            </CardTitle>
          </CardHeader>
          <CardContent>
            <div className="grid grid-cols-2 gap-4">
              <div className="flex items-center">
                <div className={`w-3 h-3 rounded-full mr-2 ${isWalletConnected ? 'bg-green-500' : 'bg-red-500'}`}></div>
                <span>Wallet Connection: {isWalletConnected ? 'Connected' : 'Disconnected'}</span>
              </div>
              
              <div className="flex items-center">
                <div className={`w-3 h-3 rounded-full mr-2 ${isBackendConnected ? 'bg-green-500' : 'bg-red-500'}`}></div>
                <span>Backend Connection: {isBackendConnected ? 'Connected' : 'Disconnected'}</span>
              </div>
              
              <div className="flex items-center">
                <div className={`w-3 h-3 rounded-full mr-2 ${isSignedIn ? 'bg-green-500' : 'bg-red-500'}`}></div>
                <span>Clerk Authentication: {isSignedIn ? 'Signed In' : 'Not Signed In'}</span>
              </div>
              
              <div className="flex items-center">
                <div className={`w-3 h-3 rounded-full mr-2 ${isGithubAuthed ? 'bg-green-500' : 'bg-red-500'}`}></div>
                <span>GitHub (Backend): {isGithubAuthed ? 'Authenticated' : 'Not Authenticated'}</span>
              </div>
            </div>
            
            <div className="mt-4">
              <div className={`p-3 rounded-md ${isFullyAuthenticated ? 'bg-green-100 text-green-800' : 'bg-amber-100 text-amber-800'}`}>
                <div className="flex items-center">
                  {isFullyAuthenticated ? (
                    <CheckCircle2 className="h-5 w-5 mr-2" />
                  ) : (
                    <AlertTriangle className="h-5 w-5 mr-2" />
                  )}
                  <span className="font-medium">
                    {isFullyAuthenticated 
                      ? 'All authentication requirements met - Ready to deploy!' 
                      : 'Complete all authentication steps to enable deployment'}
                  </span>
                </div>
              </div>
            </div>
            
            <Button 
              onClick={checkBackendConnection} 
              variant="outline" 
              size="sm" 
              className="mt-4"
            >
              Refresh Status
            </Button>
          </CardContent>
        </Card>
        
        <Tabs defaultValue="wallet">
          <TabsList>
            <TabsTrigger value="wallet">Wallet Auth</TabsTrigger>
            <TabsTrigger value="clerk">Clerk Auth</TabsTrigger>
            <TabsTrigger value="github">GitHub Sync</TabsTrigger>
            <TabsTrigger value="gql">GraphQL</TabsTrigger>
            <TabsTrigger value="deploy">Deployment</TabsTrigger>
          </TabsList>
          
          <TabsContent value="wallet">
            <h2 className="text-xl font-semibold mb-4">Wallet Authentication</h2>
            <p className="text-sm text-gray-600 mb-4">
              This authenticates your wallet with the backend for payment processing and transaction signing.
            </p>
            <DirectKeyAuth />
          </TabsContent>
          
          <TabsContent value="clerk">
            <h2 className="text-xl font-semibold mb-4">Clerk Authentication</h2>
            <p className="text-sm text-gray-600 mb-4">
              This provides GitHub authentication and user management through Clerk.
            </p>
            
            {!isSignedIn ? (
              <Card>
                <CardHeader>
                  <CardTitle>Sign In with Clerk</CardTitle>
                  <CardDescription>
                    Sign in to access GitHub repositories and user management features
                  </CardDescription>
                </CardHeader>
                <CardContent>
                  <SignIn />
                </CardContent>
              </Card>
            ) : (
              <Card>
                <CardHeader>
                  <CardTitle>Clerk Authentication Status</CardTitle>
                  <CardDescription>
                    You are signed in with Clerk
                  </CardDescription>
                </CardHeader>
                <CardContent>
                  <div className="p-3 bg-green-100 text-green-800 rounded flex items-center mb-4">
                    <CheckCircle2 className="h-5 w-5 mr-2" />
                    <span>Successfully signed in with Clerk</span>
                  </div>
                  
                  <div className="space-y-2">
                    <p><strong>User:</strong> {user?.emailAddresses[0]?.emailAddress}</p>
                    <p><strong>User ID:</strong> {user?.id}</p>
                    <p><strong>GitHub Connected:</strong> {
                      user?.externalAccounts.find(account => account.provider === 'github') 
                        ? 'Yes' : 'No'
                    }</p>
                  </div>
                  
                  {!user?.externalAccounts.find(account => account.provider === 'github') && (
                    <div className="mt-4 p-3 bg-amber-100 text-amber-800 rounded">
                      <p className="text-sm">
                        You need to connect your GitHub account in Clerk to proceed.
                      </p>
                      <Button 
                        onClick={() => window.open('https://accounts.clerk.dev/user', '_blank')}
                        className="mt-2"
                        size="sm"
                      >
                        Connect GitHub Account
                      </Button>
                    </div>
                  )}
                </CardContent>
              </Card>
            )}
          </TabsContent>
          
          <TabsContent value="github">
            <h2 className="text-xl font-semibold mb-4">GitHub Authentication</h2>
            <p className="text-sm text-gray-600 mb-4">
              This page manages two separate GitHub connections for different purposes.
            </p>
            
            <div className="space-y-6">
              {/* Clerk GitHub Integration */}
              <Card>
                <CardHeader>
                  <CardTitle>Clerk GitHub Integration</CardTitle>
                  <CardDescription>
                    Provides repository access and user management through Clerk
                  </CardDescription>
                </CardHeader>
                <CardContent>
                  <div className="space-y-4">
                    <div className="flex items-center">
                      <div className={`w-3 h-3 rounded-full mr-2 ${isSignedIn ? 'bg-green-500' : 'bg-red-500'}`}></div>
                      <span>Clerk Authentication: {isSignedIn ? 'Signed In' : 'Not Signed In'}</span>
                    </div>
                    
                    <div className="flex items-center">
                      <div className={`w-3 h-3 rounded-full mr-2 ${
                        isSignedIn && user?.externalAccounts.find(account => account.provider === 'github') 
                          ? 'bg-green-500' : 'bg-red-500'
                      }`}></div>
                      <span>GitHub Connected to Clerk: {
                        isSignedIn && user?.externalAccounts.find(account => account.provider === 'github') 
                          ? 'Yes' : 'No'
                      }</span>
                    </div>
                    
                    {repositories && repositories.length > 0 && (
                      <div>
                        <h3 className="text-md font-semibold mb-2">Available Repositories (via Clerk)</h3>
                        <div className="border rounded-md max-h-40 overflow-y-auto">
                          <ul className="divide-y">
                            {repositories.slice(0, 5).map((repo: any) => (
                              <li key={repo.id} className="p-2 text-sm">
                                <span className="font-medium">{repo.full_name}</span>
                              </li>
                            ))}
                            {repositories.length > 5 && (
                              <li className="p-2 text-sm text-gray-500">
                                ... and {repositories.length - 5} more repositories
                              </li>
                            )}
                          </ul>
                        </div>
                      </div>
                    )}
                    
                    {/* Token extraction for debugging */}
                    <div className="border-t pt-4">
                      <h3 className="text-md font-semibold mb-2">Debug: Token Extraction</h3>
                      <Button 
                        onClick={getClerkTokenForManualEntry}
                        disabled={!isSignedIn || !user?.externalAccounts.find(account => account.provider === 'github')}
                        variant="outline"
                        size="sm"
                      >
                        <GitBranch className="mr-2 h-4 w-4" />
                        Extract Clerk GitHub Token
                      </Button>
                      
                      {manualToken && (
                        <div className="mt-3 space-y-2">
                          <div className="p-2 bg-gray-100 dark:bg-gray-800 rounded font-mono text-xs break-all">
                            {manualToken.substring(0, 40)}...
                          </div>
                          <div className="text-xs text-gray-600">
                            Token extracted successfully (showing first 40 characters)
                          </div>
                        </div>
                      )}
                    </div>
                  </div>
                </CardContent>
              </Card>
              
              {/* Backend GitHub Authentication */}
              <GitHubBackendAuth 
                onAuthStatusChange={(isAuth) => setIsGithubAuthed(isAuth)}
              />
              
              {/* Status Summary */}
              <Card>
                <CardHeader>
                  <CardTitle>Authentication Summary</CardTitle>
                  <CardDescription>
                    Overview of all authentication systems
                  </CardDescription>
                </CardHeader>
                <CardContent>
                  <div className="grid grid-cols-1 gap-3">
                    <div className="flex items-center justify-between p-3 border rounded">
                      <span className="font-medium">Clerk GitHub (Repository Access)</span>
                      <div className="flex items-center">
                        <div className={`w-3 h-3 rounded-full mr-2 ${
                          isSignedIn && user?.externalAccounts.find(account => account.provider === 'github') 
                            ? 'bg-green-500' : 'bg-red-500'
                        }`}></div>
                        <span className="text-sm">
                          {isSignedIn && user?.externalAccounts.find(account => account.provider === 'github') 
                            ? 'Connected' : 'Not Connected'}
                        </span>
                      </div>
                    </div>
                    
                    <div className="flex items-center justify-between p-3 border rounded">
                      <span className="font-medium">Backend GitHub (Deployments)</span>
                      <div className="flex items-center">
                        <div className={`w-3 h-3 rounded-full mr-2 ${isGithubAuthed ? 'bg-green-500' : 'bg-red-500'}`}></div>
                        <span className="text-sm">{isGithubAuthed ? 'Connected' : 'Not Connected'}</span>
                      </div>
                    </div>
                    
                    <div className="flex items-center justify-between p-3 border rounded">
                      <span className="font-medium">Wallet Authentication</span>
                      <div className="flex items-center">
                        <div className={`w-3 h-3 rounded-full mr-2 ${isWalletConnected ? 'bg-green-500' : 'bg-red-500'}`}></div>
                        <span className="text-sm">{isWalletConnected ? 'Connected' : 'Not Connected'}</span>
                      </div>
                    </div>
                  </div>
                  
                  <div className={`mt-4 p-3 rounded-md ${isFullyAuthenticated ? 'bg-green-100 text-green-800' : 'bg-amber-100 text-amber-800'}`}>
                    <div className="flex items-center">
                      {isFullyAuthenticated ? (
                        <CheckCircle2 className="h-5 w-5 mr-2" />
                      ) : (
                        <AlertTriangle className="h-5 w-5 mr-2" />
                      )}
                      <span className="font-medium">
                        {isFullyAuthenticated 
                          ? 'All systems connected - Ready for deployment!' 
                          : 'Complete all authentication steps to enable deployment'}
                      </span>
                    </div>
                  </div>
                </CardContent>
              </Card>
            </div>
          </TabsContent>
          
          <TabsContent value="gql">
            <h2 className="text-xl font-semibold mb-4">GraphQL Testing</h2>
            <GQLTest />
          </TabsContent>
          
          <TabsContent value="deploy">
            <h2 className="text-xl font-semibold mb-4">Deployment Testing</h2>
            
            {!isFullyAuthenticated ? (
              <Card>
                <CardHeader>
                  <CardTitle>Complete Authentication Required</CardTitle>
                  <CardDescription>
                    You need to complete all authentication steps before deploying
                  </CardDescription>
                </CardHeader>
                <CardContent>
                  <div className="space-y-4">
                    <div className="grid grid-cols-1 gap-2">
                      <div className="p-3 border rounded flex items-center">
                        <div className={`w-3 h-3 rounded-full mr-2 ${isWalletConnected ? 'bg-green-500' : 'bg-red-500'}`}></div>
                        <span>Wallet Authentication: {isWalletConnected ? 'Complete' : 'Required'}</span>
                      </div>
                      
                      <div className="p-3 border rounded flex items-center">
                        <div className={`w-3 h-3 rounded-full mr-2 ${isSignedIn ? 'bg-green-500' : 'bg-red-500'}`}></div>
                        <span>Clerk Authentication: {isSignedIn ? 'Complete' : 'Required'}</span>
                      </div>
                      
                      <div className="p-3 border rounded flex items-center">
                        <div className={`w-3 h-3 rounded-full mr-2 ${isGithubAuthed ? 'bg-green-500' : 'bg-red-500'}`}></div>
                        <span>GitHub Backend Sync: {isGithubAuthed ? 'Complete' : 'Required'}</span>
                      </div>
                    </div>
                    
                    <div className="p-4 bg-blue-50 rounded-md">
                      <h3 className="text-sm font-medium text-blue-800 mb-2">Next Steps:</h3>
                      <ol className="list-decimal pl-4 text-sm text-blue-700 space-y-1">
                        {!isWalletConnected && <li>Complete wallet authentication in the Wallet Auth tab</li>}
                        {!isSignedIn && <li>Sign in with Clerk in the Clerk Auth tab</li>}
                        {!isGithubAuthed && <li>Sync GitHub token in the GitHub Sync tab</li>}
                      </ol>
                    </div>
                  </div>
                </CardContent>
              </Card>
            ) : (
              <Card>
                <CardHeader>
                  <CardTitle>Test Deployment</CardTitle>
                  <CardDescription>
                    Deploy a test project to verify deployment functionality
                  </CardDescription>
                </CardHeader>
                <CardContent>
                  <div className="space-y-4">
                    {organizations.length > 0 ? (
                      <div className="space-y-2">
                        <Label htmlFor="organization">Organization</Label>
                        <select 
                          id="organization"
                          className="w-full p-2 border rounded"
                          value={selectedOrg}
                          onChange={(e) => setSelectedOrg(e.target.value)}
                        >
                          {organizations.map(org => (
                            <option key={org.id} value={org.slug}>
                              {org.name} ({org.slug})
                            </option>
                          ))}
                        </select>
                      </div>
                    ) : (
                      <div className="p-3 bg-amber-100 text-amber-800 rounded">
                        No organizations found. You need to be part of at least one organization.
                      </div>
                    )}
                    
                    {/* Deployer Selection */}
                    <div className="space-y-2">
                      <Label htmlFor="deployer">Deployer</Label>
                      {deployersLoading ? (
                        <div className="p-2 border rounded bg-gray-50">
                          <div className="flex items-center">
                            <div className="animate-spin rounded-full h-4 w-4 border-b-2 border-gray-900 mr-2"></div>
                            Loading deployers...
                          </div>
                        </div>
                      ) : deployers.length > 0 ? (
                        <select
                          id="deployer"
                          className="w-full p-2 border rounded"
                          value={selectedDeployer}
                          onChange={(e) => setSelectedDeployer(e.target.value)}
                        >
                          <option value="">Select a deployer</option>
                          {deployers.map((deployer) => (
                            <option key={deployer.deployerLrn} value={deployer.deployerLrn}>
                              {deployer.deployerLrn} 
                              {deployer.minimumPayment && ` (Min: ${deployer.minimumPayment})`}
                            </option>
                          ))}
                        </select>
                      ) : (
                        <div className="p-3 bg-amber-100 text-amber-800 rounded">
                          <p className="text-sm">
                            No deployers available. The backend needs to have deployers configured.
                          </p>
                          <Button 
                            onClick={fetchDeployers} 
                            variant="outline" 
                            size="sm" 
                            className="mt-2"
                          >
                            Refresh Deployers
                          </Button>
                        </div>
                      )}
                    </div>
                    
                    <div className="space-y-2">
                      <Label htmlFor="name">Project Name</Label>
                      <Input
                        id="name"
                        name="name"
                        value={formData.name}
                        onChange={handleChange}
                        placeholder="test-deployment"
                      />
                    </div>
                    
                    <div className="space-y-2">
                      <Label htmlFor="repository">Repository</Label>
                      {repositories && repositories.length > 0 ? (
                        <select
                          id="repository"
                          name="repository"
                          className="w-full p-2 border rounded"
                          value={formData.repository}
                          onChange={handleChange}
                        >
                          <option value="">Select a repository</option>
                          {repositories.map((repo: any) => (
                            <option key={repo.id} value={repo.full_name}>
                              {repo.full_name}
                            </option>
                          ))}
                        </select>
                      ) : (
                        <div>
                          <p className="mb-2 text-sm text-amber-800">
                            Enter the repository manually (format: owner/repo-name)
                          </p>
                          <Input
                            id="repository"
                            name="repository"
                            value={formData.repository}
                            onChange={handleChange}
                            placeholder="username/repo-name"
                          />
                        </div>
                      )}
                    </div>
                    
                    <div className="space-y-2">
                      <Label htmlFor="branch">Branch</Label>
                      <Input
                        id="branch"
                        name="branch"
                        value={formData.branch}
                        onChange={handleChange}
                        placeholder="main or master"
                      />
                    </div>
                    
                    <Button 
                      onClick={handleDeploy} 
                      disabled={isDeploying || !selectedOrg || !formData.repository || !selectedDeployer}
                      className="w-full"
                    >
                      {isDeploying ? (
                        <>
                          <Loader2 className="mr-2 h-4 w-4 animate-spin" />
                          Deploying...
                        </>
                      ) : 'Deploy Test Project'}
                    </Button>
                    
                    {deploymentError && (
                      <div className="p-3 bg-red-100 text-red-800 rounded">
                        {deploymentError}
                      </div>
                    )}
                    
                    {deploymentResult && (
                      <div className="p-3 bg-green-100 text-green-800 rounded">
                        <div className="flex items-start">
                          <CheckCircle2 className="h-5 w-5 mr-2 mt-0.5" />
                          <div>
                            <h3 className="font-medium">{deploymentResult.message}</h3>
                            <p className="text-sm mt-1">Project ID: {deploymentResult.project?.id}</p>
                            <p className="text-sm">Name: {deploymentResult.project?.name}</p>
                            <p className="text-sm">Repository: {deploymentResult.project?.repository}</p>
                          </div>
                        </div>
                        
                        <details className="mt-2">
                          <summary className="cursor-pointer text-sm font-medium">
                            Show full project details
                          </summary>
                          <pre className="bg-white p-2 rounded mt-1 overflow-auto max-h-64 text-xs">
                            {JSON.stringify(deploymentResult.project, null, 2)}
                          </pre>
                        </details>
                      </div>
                    )}
                  </div>
                </CardContent>
              </Card>
            )}
          </TabsContent>
        </Tabs>
      </div>
      
      <div className="mt-8 p-4 bg-gray-100 dark:bg-gray-800 rounded-md">
        <h2 className="text-lg font-semibold mb-2">Hybrid Authentication Flow</h2>
        <p className="mb-2 text-sm">
          This deployment system requires both wallet and GitHub authentication:
        </p>
        <div className="grid grid-cols-1 md:grid-cols-2 gap-4 mt-4">
          <div>
            <h3 className="text-md font-semibold mb-2">Wallet Authentication (DirectKeyAuth)</h3>
            <ul className="list-disc pl-5 space-y-1 text-sm">
              <li>Provides Ethereum wallet connection</li>
              <li>Enables transaction signing for payments</li>
              <li>Required for deployment costs and blockchain operations</li>
            </ul>
          </div>
          <div>
            <h3 className="text-md font-semibold mb-2">GitHub Authentication (Clerk)</h3>
            <ul className="list-disc pl-5 space-y-1 text-sm">
              <li>Provides access to GitHub repositories</li>
              <li>Enables repository cloning and deployment</li>
              <li>Required for backend deployment operations</li>
            </ul>
          </div>
        </div>
      </div>
    </PageWrapper>
  )
}