// src/components/SIWEAuth.tsx with raw signature approach
'use client'
import { useState, useEffect } from 'react'
import { useWallet } from '@/context/WalletContext'
import { Button } from '@workspace/ui/components/button'
import { CheckBalanceWrapper } from './iframe/check-balance-iframe/CheckBalanceWrapper'
import { CopyIcon } from 'lucide-react'

// Generate a random nonce
function generateNonce() {
  return Math.random().toString(36).slice(2) + Math.random().toString(36).slice(2);
}

export function SIWEAuth() {
  const { wallet, isConnected, connect, disconnect } = useWallet()
  const [sessionStatus, setSessionStatus] = useState<'checking' | 'authenticated' | 'unauthenticated'>('checking')
  const [sessionData, setSessionData] = useState<any>(null)
  const [isAuthenticating, setIsAuthenticating] = useState(false)
  const [authError, setAuthError] = useState<string | null>(null)
  const [signedMessage, setSignedMessage] = useState<string | null>(null)
  const [messageToSign, setMessageToSign] = useState<string | null>(null)
  const [debugInfo, setDebugInfo] = useState<string>('')
  
  // Check if we already have a session
  const checkSession = async () => {
    try {
      setSessionStatus('checking')
      const response = await fetch('http://localhost:8000/auth/session', {
        method: 'GET',
        credentials: 'include',
      })
      
      if (response.ok) {
        const data = await response.json()
        setSessionStatus('authenticated')
        setSessionData(data)
        console.log('Session check successful:', data)
      } else {
        setSessionStatus('unauthenticated')
        setSessionData(null)
        console.log('Session check failed:', await response.text())
      }
    } catch (error) {
      console.error('Error checking session:', error)
      setSessionStatus('unauthenticated')
      setSessionData(null)
    }
  }
  
  // Check session on component mount
  useEffect(() => {
    checkSession()
  }, [])

  // Copy text to clipboard
  const copyToClipboard = (text: string) => {
    navigator.clipboard.writeText(text)
      .then(() => {
        console.log('Text copied to clipboard')
      })
      .catch(err => {
        console.error('Could not copy text: ', err)
      })
  }

  // Create a SIWE message with the correct Ethereum address
  const createSiweMessage = () => {
    // We want to try both our displayed address and the expected one from errors
    // We'll use the displayed address by default
    const ethAddress = '0x1ac42F4A25Ae0137d10a825a2e33e32de0F6B57E';
    
    const domain = window.location.host
    const origin = window.location.origin
    const chainId = 1 // Ethereum mainnet
    const statement = 'Sign in With Ethereum.'
    const nonce = generateNonce()
    const issuedAt = new Date().toISOString()
    
    // IMPORTANT: This format must exactly match what the SiweMessage constructor expects
    return `${domain} wants you to sign in with your Ethereum account:
${ethAddress}

${statement}

URI: ${origin}
Version: 1
Chain ID: ${chainId}
Nonce: ${nonce}
Issued At: ${issuedAt}`
  }

  // Generate the message for signing
  const generateMessageToSign = async () => {
    if (!wallet?.address) {
      setAuthError('Wallet not connected')
      return
    }

    try {
      setIsAuthenticating(true)
      setAuthError(null)
      
      // Create a SIWE message with the Ethereum address
      const message = createSiweMessage()
      console.log('SIWE Message with Ethereum address:', message)
      setDebugInfo(`Generated message with Ethereum address. IMPORTANT: Make sure "Ethereum" is selected in the wallet dropdown when signing.`)
      
      // Set the message to sign
      setMessageToSign(message)
      
    } catch (error) {
      console.error('Error generating message:', error)
      setAuthError(`Error: ${error instanceof Error ? error.message : String(error)}`)
    } finally {
      setIsAuthenticating(false)
    }
  }

  // Check auth without sending signature
  const checkAuthWithoutSignature = async () => {
    try {
      setIsAuthenticating(true)
      setAuthError(null)
      setDebugInfo('Trying auth without signature...')
      
      // Create API route to handle this
      const response = await fetch('/api/dev-auth', {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify({
          address: '0x1ac42F4A25Ae0137d10a825a2e33e32de0F6B57E'
        })
      })
      
      const data = await response.json()
      console.log('Dev auth response:', data)
      setDebugInfo(prev => `${prev}\nDev auth response: ${JSON.stringify(data)}`)
      
      if (response.ok && data.success) {
        console.log('Dev auth successful!')
        setDebugInfo(prev => `${prev}\nDev auth successful!`)
        await checkSession()
      } else {
        throw new Error(`Dev auth failed: ${JSON.stringify(data)}`)
      }
      
    } catch (error) {
      console.error('Dev auth error:', error)
      setAuthError(`Error: ${error instanceof Error ? error.message : String(error)}`)
    } finally {
      setIsAuthenticating(false)
    }
  }

  // Submit signature to validate
  const submitSignature = async () => {
    if (!messageToSign || !signedMessage) {
      setAuthError('Missing message or signature')
      return
    }

    try {
      setIsAuthenticating(true)
      setAuthError(null)
      setDebugInfo(prev => `${prev}\nSubmitting raw signature...`)
      
      // Log the original signature
      console.log('Raw signature:', signedMessage)
      setDebugInfo(prev => `${prev}\nRaw signature: ${signedMessage}`)
      
      // Try using the raw signature directly
      const response = await fetch('http://localhost:8000/auth/validate', {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        credentials: 'include',
        body: JSON.stringify({
          message: messageToSign,
          signature: signedMessage
        })
      })
      
      let responseData = {}
      try {
        responseData = await response.json()
      } catch (e) {
        console.log('Error parsing response:', e)
      }
      
      console.log('Validation response:', responseData)
      setDebugInfo(prev => `${prev}\nValidation response: ${JSON.stringify(responseData)}`)
      
      // If successful, we're done
      if (response.ok && responseData.success) {
        console.log('Authentication successful!')
        setDebugInfo(prev => `${prev}\nAuthentication successful!`)
        
        // Clear message and signature
        setMessageToSign(null)
        setSignedMessage(null)
        
        // Check if we now have a session
        await checkSession()
        return
      }
      
      // If we get here, it failed
      throw new Error(`Validation failed: ${JSON.stringify(responseData)}`)
      
    } catch (error) {
      console.error('Authentication error:', error)
      setAuthError(`Error: ${error instanceof Error ? error.message : String(error)}`)
      setSessionStatus('unauthenticated')
    } finally {
      setIsAuthenticating(false)
    }
  }

  return (
    <div className="p-4 border rounded-md">
      {/* Hidden iframe for wallet connection */}
      <CheckBalanceWrapper />
      
      <h2 className="text-lg font-bold mb-4">Sign-In With Ethereum</h2>
      
      <div className="mb-4">
        <h3 className="text-md font-semibold mb-2">Wallet Status</h3>
        <p className="mb-2">
          Status: <span className={isConnected ? "text-green-500" : "text-red-500"}>
            {isConnected ? 'Connected' : 'Disconnected'}
          </span>
        </p>
        {wallet && wallet.address && (
          <div className="p-2 bg-gray-800 text-white rounded mb-2">
            <p className="font-mono text-sm break-all">Laconic Address: {wallet.address}</p>
            <p className="font-mono text-sm break-all mt-1">Ethereum Address: 0x1ac42F4A25Ae0137d10a825a2e33e32de0F6B57E</p>
          </div>
        )}
        
        <div className="mt-2">
          {!isConnected ? (
            <Button onClick={connect}>Connect Wallet</Button>
          ) : (
            <Button variant="outline" onClick={disconnect}>Disconnect</Button>
          )}
        </div>
      </div>
      
      {isConnected && sessionStatus !== 'authenticated' && (
        <div className="mb-4">
          <h3 className="text-md font-semibold mb-2">Authentication</h3>
          
          <div className="p-3 bg-amber-100 text-amber-800 rounded mb-4">
            <p className="font-semibold text-sm">IMPORTANT:</p>
            <p className="text-sm">When signing the message, make sure "Ethereum" is selected in the wallet's network dropdown.</p>
          </div>
          
          {!messageToSign ? (
            <div className="space-y-4">
              <Button 
                onClick={generateMessageToSign} 
                disabled={isAuthenticating}
                className="w-full"
              >
                Generate SIWE Message
              </Button>
              
              <div className="border-t pt-4">
                <p className="text-sm mb-2 text-amber-700 font-semibold">
                  Alternative Authentication Methods
                </p>
                <Button 
                  onClick={checkAuthWithoutSignature}
                  disabled={isAuthenticating}
                  variant="outline"
                  className="w-full"
                >
                  Try Development Authentication
                </Button>
                <p className="text-xs text-gray-500 mt-1">
                  This will try to create a session using a development-only endpoint.
                </p>
              </div>
            </div>
          ) : (
            <div className="mb-4">
              <div className="p-3 bg-gray-800 text-white rounded mb-3">
                <div className="flex items-center justify-between mb-2">
                  <h4 className="text-sm font-semibold">Message to Sign:</h4>
                  <Button 
                    size="sm" 
                    variant="ghost" 
                    onClick={() => copyToClipboard(messageToSign)}
                    className="h-8 px-2"
                  >
                    <CopyIcon className="h-4 w-4 mr-1" /> Copy
                  </Button>
                </div>
                <pre className="font-mono text-xs whitespace-pre-wrap break-all">{messageToSign}</pre>
              </div>
              
              <div className="mb-3">
                <p className="text-sm mb-2">
                  1. Copy the message above
                </p>
                <p className="text-sm mb-2">
                  2. Go to your wallet's "Sign Message" page 
                  (<a href="http://localhost:4000/SignMessage" target="_blank" className="text-blue-500 underline">
                    Open Wallet Sign Page
                  </a>)
                </p>
                <p className="text-sm mb-2 font-medium text-amber-700">
                  3. Make sure "Ethereum" is selected in the network dropdown
                </p>
                <p className="text-sm mb-2">
                  4. Paste the message and sign it
                </p>
                <p className="text-sm mb-2">
                  5. Copy the ENTIRE signature and paste it below
                </p>
              </div>
              
              <div className="mb-3">
                <h4 className="text-sm font-semibold mb-2">Paste Signature:</h4>
                <textarea 
                  className="w-full p-2 border rounded dark:bg-gray-800 dark:text-white" 
                  rows={3}
                  value={signedMessage || ''}
                  onChange={(e) => setSignedMessage(e.target.value)}
                  placeholder="Paste signature here (including 'Signature' prefix)"
                />
              </div>
              
              <Button 
                onClick={submitSignature} 
                disabled={isAuthenticating || !signedMessage}
                className="mb-2"
              >
                {isAuthenticating ? 'Validating...' : 'Validate Signature'}
              </Button>
              
              <Button 
                variant="outline"
                onClick={() => {
                  setMessageToSign(null);
                  setSignedMessage(null);
                  setDebugInfo('');
                }}
                className="ml-2 mb-2"
              >
                Cancel
              </Button>
            </div>
          )}
          
          {debugInfo && (
            <div className="mt-4 p-2 bg-gray-800 text-white rounded">
              <h4 className="text-sm font-semibold mb-2">Debug Information:</h4>
              <pre className="font-mono text-xs whitespace-pre-wrap">{debugInfo}</pre>
            </div>
          )}
          
          {authError && (
            <div className="mt-2 p-2 bg-red-100 border border-red-300 text-red-800 rounded whitespace-pre-line">
              {authError}
            </div>
          )}
        </div>
      )}
      
      <div className="mb-4">
        <h3 className="text-md font-semibold mb-2">Backend Session</h3>
        <p className="mb-2">
          Status: 
          <span className={
            sessionStatus === 'authenticated' ? "text-green-500" : 
            sessionStatus === 'unauthenticated' ? "text-red-500" : 
            "text-yellow-500"
          }>
            {' '}{sessionStatus}
          </span>
        </p>
        
        {sessionData && (
          <div className="p-2 bg-gray-800 text-white rounded mb-2">
            <pre className="font-mono text-sm overflow-auto max-h-32">{JSON.stringify(sessionData, null, 2)}</pre>
          </div>
        )}
        
        <div className="mt-2">
          <Button variant="outline" onClick={checkSession}>Check Session</Button>
        </div>
      </div>
      
      <div className="mt-4 p-3 bg-blue-50 text-blue-800 rounded text-sm">
        <p className="font-semibold mb-1">About Laconic Wallet Authentication:</p>
        <p className="mt-2 text-xs">
          The Laconic wallet supports multiple networks including Ethereum. For SIWE authentication, you must:
        </p>
        <ol className="list-decimal text-xs mt-1 pl-4">
          <li>Use your Ethereum address in the sign-in message</li>
          <li>Make sure "Ethereum" is selected in the network dropdown when signing</li>
          <li>The signature will then be created with your Ethereum private key</li>
        </ol>
      </div>
    </div>
  )
}