// src/components/GitHubBackendAuth.tsx
'use client'
import { useState, useEffect } from 'react'
import { Button } from '@workspace/ui/components/button'
import { Card, CardContent, CardDescription, CardHeader, CardTitle } from '@workspace/ui/components/card'
import { CheckCircle2, GitBranch, ExternalLink, AlertCircle } from 'lucide-react'
import { toast } from 'sonner'
import { useGQLClient } from '@/context'

interface GitHubBackendAuthProps {
  onAuthStatusChange?: (isAuthenticated: boolean) => void
}

export function GitHubBackendAuth({ onAuthStatusChange }: GitHubBackendAuthProps) {
  const [isAuthenticated, setIsAuthenticated] = useState(false)
  const [isLoading, setIsLoading] = useState(false)
  const [isChecking, setIsChecking] = useState(true)
  const gqlClient = useGQLClient()
  
  // GitHub OAuth configuration - replace with your backend OAuth app credentials
  const GITHUB_CLIENT_ID = process.env.NEXT_PUBLIC_GITHUB_BACKEND_CLIENT_ID || 'your_backend_client_id_here'
  const REDIRECT_URI = `${window.location.origin}/auth/github/backend-callback`
  
  // Check current authentication status
  const checkAuthStatus = async () => {
    try {
      setIsChecking(true)
      const userData = await gqlClient.getUser()
      const hasGitHubToken = !!userData.user.gitHubToken
      setIsAuthenticated(hasGitHubToken)
      onAuthStatusChange?.(hasGitHubToken)
    } catch (error) {
      console.error('Error checking GitHub auth status:', error)
      setIsAuthenticated(false)
      onAuthStatusChange?.(false)
    } finally {
      setIsChecking(false)
    }
  }
  
  // Check auth status on mount
  useEffect(() => {
    checkAuthStatus()
  }, [])
  
  // Listen for OAuth callback completion
  useEffect(() => {
    const handleAuthComplete = (event: MessageEvent) => {
      if (event.origin !== window.location.origin) return
      
      if (event.data.type === 'GITHUB_BACKEND_AUTH_SUCCESS') {
        toast.success('GitHub backend authentication successful!')
        checkAuthStatus()
      } else if (event.data.type === 'GITHUB_BACKEND_AUTH_ERROR') {
        toast.error(`GitHub authentication failed: ${event.data.message}`)
        setIsLoading(false)
      }
    }
    
    window.addEventListener('message', handleAuthComplete)
    return () => window.removeEventListener('message', handleAuthComplete)
  }, [])
  
  const startGitHubAuth = () => {
    setIsLoading(true)
    
    // Generate state parameter for security
    const state = Math.random().toString(36).substring(2, 15)
    sessionStorage.setItem('github_oauth_state', state)
    
    // Build GitHub OAuth URL
    const params = new URLSearchParams({
      client_id: GITHUB_CLIENT_ID,
      redirect_uri: REDIRECT_URI,
      scope: 'repo public_repo user:email',
      state: state,
      response_type: 'code'
    })
    
    const authUrl = `https://github.com/login/oauth/authorize?${params.toString()}`
    
    // Open OAuth in popup window
    const popup = window.open(
      authUrl,
      'github-oauth',
      'width=600,height=700,scrollbars=yes,resizable=yes'
    )
    
    // Monitor popup closure
    const checkClosed = setInterval(() => {
      if (popup?.closed) {
        clearInterval(checkClosed)
        setIsLoading(false)
      }
    }, 1000)
  }
  
  const disconnectGitHub = async () => {
    try {
      setIsLoading(true)
      await gqlClient.unauthenticateGithub()
      setIsAuthenticated(false)
      onAuthStatusChange?.(false)
      toast.success('GitHub disconnected successfully')
    } catch (error) {
      console.error('Error disconnecting GitHub:', error)
      toast.error('Failed to disconnect GitHub')
    } finally {
      setIsLoading(false)
    }
  }
  
  if (isChecking) {
    return (
      <Card>
        <CardHeader>
          <CardTitle>GitHub Backend Authentication</CardTitle>
          <CardDescription>Checking authentication status...</CardDescription>
        </CardHeader>
        <CardContent>
          <div className="flex items-center justify-center p-4">
            <div className="animate-spin rounded-full h-6 w-6 border-b-2 border-gray-900"></div>
          </div>
        </CardContent>
      </Card>
    )
  }
  
  return (
    <Card>
      <CardHeader>
        <CardTitle className="flex items-center">
          <GitBranch className="mr-2 h-5 w-5" />
          GitHub Backend Authentication
        </CardTitle>
        <CardDescription>
          Connect your GitHub account to the backend for deployment operations
        </CardDescription>
      </CardHeader>
      <CardContent>
        <div className="space-y-4">
          <div className="flex items-center">
            <div className={`w-3 h-3 rounded-full mr-2 ${
              isAuthenticated ? 'bg-green-500' : 'bg-red-500'
            }`}></div>
            <span>
              Backend GitHub Token: {isAuthenticated ? 'Connected' : 'Not Connected'}
            </span>
          </div>
          
          {!isAuthenticated ? (
            <div className="space-y-3">
              <div className="p-3 bg-blue-50 dark:bg-blue-900/20 rounded-md">
                <div className="flex items-start">
                  <AlertCircle className="h-5 w-5 text-blue-600 mr-2 mt-0.5" />
                  <div className="text-sm text-blue-800 dark:text-blue-200">
                    <p className="font-medium mb-1">Backend GitHub Authentication Required</p>
                    <p>
                      This connects your GitHub account directly to the backend for deployment operations. 
                      This is separate from your Clerk GitHub integration.
                    </p>
                  </div>
                </div>
              </div>
              
              <Button 
                onClick={startGitHubAuth}
                disabled={isLoading}
                className="w-full"
              >
                {isLoading ? (
                  <>
                    <div className="animate-spin rounded-full h-4 w-4 border-b-2 border-white mr-2"></div>
                    Connecting...
                  </>
                ) : (
                  <>
                    <ExternalLink className="mr-2 h-4 w-4" />
                    Connect GitHub to Backend
                  </>
                )}
              </Button>
              
              <div className="text-xs text-gray-500 text-center">
                This will open GitHub in a popup window for authentication
              </div>
            </div>
          ) : (
            <div className="space-y-3">
              <div className="p-3 bg-green-50 dark:bg-green-900/20 rounded-md">
                <div className="flex items-center">
                  <CheckCircle2 className="h-5 w-5 text-green-600 mr-2" />
                  <div className="text-sm text-green-800 dark:text-green-200">
                    <p className="font-medium">GitHub Backend Connected Successfully</p>
                    <p>Your backend can now access GitHub for deployments</p>
                  </div>
                </div>
              </div>
              
              <div className="flex space-x-2">
                <Button 
                  onClick={checkAuthStatus}
                  variant="outline"
                  size="sm"
                  className="flex-1"
                >
                  Refresh Status
                </Button>
                
                <Button 
                  onClick={disconnectGitHub}
                  variant="outline"
                  size="sm"
                  className="flex-1"
                  disabled={isLoading}
                >
                  {isLoading ? 'Disconnecting...' : 'Disconnect'}
                </Button>
              </div>
            </div>
          )}
          
          {!GITHUB_CLIENT_ID.startsWith('your_') ? null : (
            <div className="p-3 bg-amber-50 dark:bg-amber-900/20 rounded-md">
              <div className="flex items-start">
                <AlertCircle className="h-5 w-5 text-amber-600 mr-2 mt-0.5" />
                <div className="text-sm text-amber-800 dark:text-amber-200">
                  <p className="font-medium mb-1">Configuration Required</p>
                  <p>
                    Please set <code>NEXT_PUBLIC_GITHUB_BACKEND_CLIENT_ID</code> in your environment variables
                    with your backend GitHub OAuth app client ID.
                  </p>
                </div>
              </div>
            </div>
          )}
        </div>
      </CardContent>
    </Card>
  )
}