import type React from 'react'
import type { DropdownProps } from './types'

/**
 * A simple dropdown component using the native select element.
 *
 * @component
 * @param {DropdownProps} props - The props for the Dropdown component.
 * @returns {React.ReactElement} A dropdown element.
 *
 * @example
 * ```tsx
 * <Dropdown
 *   options={[{ value: '1', label: 'One' }, { value: '2', label: 'Two' }]}
 *   onChange={(option) => console.log(option)}
 *   placeholder="Select an option"
 * />
 * ```
 */
export const Dropdown = ({
  placeholder,
  options,
  onChange,
  value
}: DropdownProps) => {
  const handleChange = (event: React.ChangeEvent<HTMLSelectElement>) => {
    const selectedOption = options.find(
      (option) => option.value === event.target.value
    )
    if (selectedOption) {
      onChange(selectedOption)
    }
  }

  return (
    <select
      className="w-full px-3 py-2 border rounded appearance-none"
      value={value?.value || ''}
      onChange={handleChange}
      aria-label={placeholder}
    >
      {!value && placeholder && (
        <option value="" disabled>
          {placeholder}
        </option>
      )}
      {options.map((option) => (
        <option key={option.value} value={option.value}>
          {option.label}
        </option>
      ))}
    </select>
  )
}
