import React, { useEffect, useRef, useState } from 'react'
import { FormatMilliSecond } from '../format-milli-second'
import type { StopwatchProps } from './types'

export const setStopWatchOffset = (time: string) => {
  const providedTime = new Date(time)
  const currentTime = new Date()
  const timeDifference = currentTime.getTime() - providedTime.getTime()
  currentTime.setMilliseconds(currentTime.getMilliseconds() + timeDifference)
  return currentTime
}

/**
 * A stopwatch component that tracks elapsed time.
 *
 * @component
 * @param {StopwatchProps} props - The props for the Stopwatch component.
 * @returns {React.ReactElement} A stopwatch element.
 *
 * @example
 * ```tsx
 * <StopWatch offsetTimestamp={new Date()} isPaused={false} />
 * ```
 */
export const StopWatch = ({
  offsetTimestamp,
  isPaused,
  ...props
}: StopwatchProps) => {
  const [elapsedTime, setElapsedTime] = useState(0)
  const intervalRef = useRef<number | null>(null)
  const startTimeRef = useRef(offsetTimestamp.getTime())

  // Set start time when offsetTimestamp changes
  useEffect(() => {
    startTimeRef.current = offsetTimestamp.getTime()
  }, [offsetTimestamp])

  // Handle timer start/stop based on isPaused state
  useEffect(() => {
    // Clear any existing interval
    if (intervalRef.current !== null) {
      window.clearInterval(intervalRef.current)
      intervalRef.current = null
    }

    if (!isPaused) {
      // Start the timer
      intervalRef.current = window.setInterval(() => {
        const now = Date.now()
        const elapsed = now - startTimeRef.current
        setElapsedTime(elapsed)
      }, 1000) // Update every second
    }

    // Cleanup on unmount
    return () => {
      if (intervalRef.current !== null) {
        window.clearInterval(intervalRef.current)
      }
    }
  }, [isPaused]) // Only re-run when isPaused changes

  return <FormatMilliSecond time={elapsedTime} {...props} />
}
