import type React from 'react'
import type { ISeparator, IStep, IStepperNavProps } from './types'

/**
 * A navigation component for displaying steps in a vertical layout.
 *
 * @component
 * @param {IStepperNavProps} props - The props for the StepperNav component.
 * @returns {React.ReactElement} A stepper navigation element.
 *
 * @example
 * ```tsx
 * <StepperNav steps={[{ stepContent: () => <div>Step 1</div> }]} />
 * ```
 */
export const StepperNav = (props: IStepperNavProps): JSX.Element => {
  return (
    <nav>
      {props.steps.map(
        (
          { stepContent, stepStateColor, onClickHandler, stepStatusCircleSize },
          stepIndex
        ) => (
          <div key={`step-${stepContent().toString()}-${stepIndex}`}>
            <Step
              stepContent={stepContent}
              statusColor={stepStateColor}
              onClickHandler={onClickHandler}
              statusCircleSize={stepStatusCircleSize}
            />
            {stepIndex !== props.steps.length - 1 && (
              <div
                style={{
                  paddingLeft: `${Math.floor((stepStatusCircleSize ?? 16) / 2) + 1}px`
                }}
              >
                <Separator />
              </div>
            )}
          </div>
        )
      )}
    </nav>
  )
}

/**
 * A separator component for the vertical stepper.
 *
 * @component
 * @param {ISeparator} props - The props for the Separator component.
 * @returns {React.ReactElement} A separator element.
 */
export const Separator = ({ height }: ISeparator): JSX.Element => {
  return (
    <div
      className="h-[5vh] w-0.5 border border-gray-200 bg-gray-200"
      style={{ height: height ?? '5vh' }}
    />
  )
}

/**
 * A step component for the vertical stepper.
 *
 * @component
 * @param {IStep} props - The props for the Step component.
 * @returns {React.ReactElement} A step element.
 */
export const Step = ({
  stepContent,
  statusColor,
  statusCircleSize,
  onClickHandler
}: IStep): JSX.Element => {
  const handleKeyDown = (event: React.KeyboardEvent<HTMLButtonElement>) => {
    if (event.key === 'Enter' || event.key === ' ') {
      onClickHandler?.()
    }
  }

  return (
    <button
      type="button"
      onClick={onClickHandler}
      onKeyDown={handleKeyDown}
      className="inline-flex flex-wrap gap-3 p-0.5 cursor-pointer"
    >
      <div>
        <div
          className="rounded-full border-2 border-gray-200"
          style={{
            width: statusCircleSize ?? 16,
            height: statusCircleSize ?? 16,
            background: statusColor ?? 'white',
            borderRadius: '50%'
          }}
        />
      </div>
      <div className="pb-0.5">{stepContent()}</div>
    </button>
  )
}
