'use client'

import { cn } from '@workspace/ui/lib/utils'
import type { ReactNode } from 'react'

/**
 * Props for the NavigationWrapper component
 * @remarks
 * Configuration interface for NavigationWrapper, a layout component that provides:
 * - Full-height, full-width container with flex column layout
 * - Support for top navigation bar (currently commented out)
 * - Flexible content area for page content
 * - Customizable styling through className prop
 *
 * @see {@link NavigationWrapper} for the component implementation
 */
export interface NavigationWrapperProps {
  /**
   * Main content for the navigation wrapper
   * @remarks
   * - Typically contains {@link PageWrapper} components
   * - Rendered in the main content area below navigation
   * - Can include any valid React nodes
   * - Takes up remaining vertical space
   *
   * @example
   * ```tsx
   * // Basic usage with PageWrapper
   * <NavigationWrapper>
   *   <PageWrapper header={{ title: "Dashboard" }}>
   *     <DashboardContent />
   *   </PageWrapper>
   * </NavigationWrapper>
   *
   * // Multiple pages in tabs/routes
   * <NavigationWrapper>
   *   {selectedTab === 'dashboard' && (
   *     <PageWrapper header={{ title: "Dashboard" }}>
   *       <DashboardContent />
   *     </PageWrapper>
   *   )}
   *   {selectedTab === 'settings' && (
   *     <PageWrapper header={{ title: "Settings" }}>
   *       <SettingsContent />
   *     </PageWrapper>
   *   )}
   * </NavigationWrapper>
   * ```
   */
  children: ReactNode

  /**
   * Optional CSS classes for the wrapper
   * @remarks
   * - Applied to the wrapper's root container
   * - Combined with default classes using the cn utility
   * - Default classes: 'flex flex-col min-h-screen w-full'
   *
   * @example
   * ```tsx
   * // Custom background
   * className="bg-background"
   *
   * // Custom max width
   * className="max-w-7xl mx-auto"
   *
   * // Custom padding
   * className="px-4 md:px-6"
   * ```
   */
  className?: string
}

/**
 * A layout component that provides navigation structure and content organization.
 *
 * @description
 * NavigationWrapper is a foundational layout component that:
 * - Creates a full-height, full-width container
 * - Supports top navigation (implementation commented out)
 * - Provides flexible content area for page content
 * - Typically wraps {@link PageWrapper} components
 *
 * @keywords layout, navigation, container, foundation-component
 * @category Layout
 * @scope Foundation
 *
 * @usage
 * Common patterns:
 *
 * Basic app layout:
 * ```tsx
 * // In app/layout.tsx
 * export default function RootLayout({ children }) {
 *   return (
 *     <html lang="en">
 *       <body>
 *         <NavigationWrapper>
 *           {children}
 *         </NavigationWrapper>
 *       </body>
 *     </html>
 *   )
 * }
 * ```
 *
 * With custom styling:
 * ```tsx
 * <NavigationWrapper className="bg-background">
 *   <PageWrapper>
 *     <Content />
 *   </PageWrapper>
 * </NavigationWrapper>
 * ```
 *
 * With route-based content:
 * ```tsx
 * <NavigationWrapper>
 *   <Routes>
 *     <Route
 *       path="/dashboard"
 *       element={
 *         <PageWrapper header={{ title: "Dashboard" }}>
 *           <DashboardContent />
 *         </PageWrapper>
 *       }
 *     />
 *     <Route
 *       path="/settings"
 *       element={
 *         <PageWrapper header={{ title: "Settings" }}>
 *           <SettingsContent />
 *         </PageWrapper>
 *       }
 *     />
 *   </Routes>
 * </NavigationWrapper>
 * ```
 *
 * @example
 * ```tsx
 * // Basic usage
 * <NavigationWrapper>
 *   <PageWrapper>
 *     <div>Page content</div>
 *   </PageWrapper>
 * </NavigationWrapper>
 *
 * // With custom styling
 * <NavigationWrapper className="bg-background">
 *   <PageWrapper>
 *     <div>Styled content</div>
 *   </PageWrapper>
 * </NavigationWrapper>
 * ```
 *
 * @param props - Component props
 * @param props.children - Main content to be rendered within the wrapper
 * @param props.className - Additional CSS classes for the root container
 *
 * @returns A layout wrapper with navigation structure
 *
 * @related {@link PageWrapper} - Commonly wrapped by NavigationWrapper
 * @composition Uses {@link cn} for class name merging
 *
 * @cssUtilities
 * - flex-col: Column layout
 * - min-h-screen: Minimum full viewport height
 * - w-full: Full width
 *
 * @accessibility
 * - Maintains semantic HTML structure
 * - Preserves content hierarchy
 * - Supports keyboard navigation (when nav is implemented)
 *
 * @performance
 * - Minimal DOM nesting
 * - Uses utility classes for styling
 * - Conditional navigation rendering
 */
export default function NavigationWrapper({
  children,
  className
}: NavigationWrapperProps) {
  return (
    <div className={cn('flex flex-col min-h-screen w-full', className)}>
      {/* Top Navigation */}
      {/* <header className="sticky top-0 z-40 w-full border-b bg-background">
               <div className="container flex h-16 items-center">
                 <div className="mr-4 flex">
                   <Link href="/" className="flex items-center space-x-2">
                     <span className="font-bold text-xl">App Logo</span>
                   </Link>
                 </div>
                 <nav className="flex items-center space-x-4 lg:space-x-6 mx-6">
                   <Link
                     href="/dashboard"
                     className="text-sm font-medium transition-colors hover:text-primary"
                   >
                     Dashboard
                   </Link>
                   <Link
                     href="/projects"
                     className="text-sm font-medium transition-colors hover:text-primary"
                   >
                     Projects
                   </Link>
                   <Link
                     href="/settings"
                     className="text-sm font-medium transition-colors hover:text-primary"
                   >
                     Settings
                   </Link>
                 </nav>
                 <div className="ml-auto flex items-center space-x-4">
                   <Button className="text-sm font-medium">Profile</Button>
                 </div>
               </div>
              </header> */}

      {/* Main Content */}
      {children}
    </div>
  )
}
