'use client'

import { Button } from '@workspace/ui/components/button'
import {
  DropdownMenu,
  DropdownMenuContent,
  DropdownMenuItem,
  DropdownMenuTrigger
} from '@workspace/ui/components/dropdown-menu'
import { cn } from '@workspace/ui/lib/utils'
import { MoreVertical } from 'lucide-react'
import Link from 'next/link'
import type { ReactNode } from 'react'

/**
 * Configuration for header action buttons/links
 * @remarks
 * Interactive elements with configurable styling and behavior:
 * - Use onClick for JS actions OR href for navigation (not both)
 * - Multiple visual styles via variant prop
 * - Optional primary emphasis for main call-to-action
 */
export interface PageAction {
  /**
   * Display text for the action button/link
   * @remarks Shown as the button/link text content
   */
  label: string

  /**
   * Visual style variant for the button
   * @remarks
   * Available styles:
   * - `default`: Standard appearance
   * - `destructive`: Dangerous actions
   * - `outline`: Bordered, transparent bg
   * - `secondary`: Less prominent
   * - `ghost`: Minimal styling
   * - `link`: Hyperlink style
   * @default 'default'
   */
  variant?:
    | 'default' // Standard appearance
    | 'destructive' // Dangerous actions
    | 'outline' // Bordered, transparent bg
    | 'secondary' // Less prominent
    | 'ghost' // Minimal styling
    | 'link' // Hyperlink style

  /**
   * Click handler for button-based actions
   * @remarks
   * - Use for JavaScript-triggered actions
   * - Mutually exclusive with `href`
   */
  onClick?: () => void

  /**
   * URL for link-based actions
   * @remarks
   * - Use for navigation to new URLs
   * - Mutually exclusive with `onClick`
   */
  href?: string

  /**
   * Whether this action should have primary visual emphasis
   * @remarks
   * - When true, applies prominent styling
   * - Useful for main call-to-action buttons
   * - Affects mobile layout (primary actions shown, secondary in dropdown)
   * @default false
   */
  isPrimary?: boolean
}

/**
 * Props for the PageHeader component
 * @remarks
 * Configuration interface for PageHeader, providing:
 * - Required title as main heading
 * - Optional subtitle for additional context
 * - Optional action buttons/links
 * - Responsive layout with mobile optimization
 * - Customizable styling
 */
export interface PageHeaderProps {
  /**
   * Main heading text
   * @remarks
   * - Rendered as h1 element
   * - Responsive text size (2xl on mobile, 30px on desktop)
   * - Bold weight with consistent line height
   */
  title: string

  /**
   * Additional content below the title
   * @remarks
   * - Can be plain text or custom component
   * - Text is muted and slightly smaller
   * - Components receive full width
   *
   * @example
   * ```tsx
   * // Text subtitle
   * subtitle="Optional description"
   *
   * // Component subtitle
   * subtitle={<SearchInput onChange={handleSearch} />}
   * ```
   */
  subtitle?: string | ReactNode

  /**
   * Array of action buttons/links
   * @remarks
   * - Desktop: All actions shown in a row
   * - Mobile: Primary actions shown, secondary in dropdown
   * - Actions can be buttons (onClick) or links (href)
   * - Support multiple visual styles via variant prop
   *
   * @see {@link PageAction} for detailed action configuration
   *
   * @example
   * ```tsx
   * actions={[
   *   {
   *     label: "Create New",
   *     isPrimary: true,
   *     onClick: () => setOpen(true)
   *   },
   *   {
   *     label: "View All",
   *     href: "/items",
   *     variant: "outline"
   *   }
   * ]}
   * ```
   */
  actions?: PageAction[]

  /**
   * Optional CSS classes
   * @remarks
   * - Applied to the header's root container
   * - Combined with default classes using cn utility
   * - Default max-width of 1232px with auto margins
   */
  className?: string
}

/**
 * A responsive page header component with title, subtitle, and actions.
 *
 * @description
 * PageHeader provides a consistent header structure with:
 * - Prominent title as h1
 * - Optional subtitle or custom component
 * - Configurable action buttons/links
 * - Responsive layout with mobile optimization
 * - Customizable styling
 *
 * @keywords header, page-title, action-buttons, responsive-header, foundation-component
 * @category Layout
 * @scope Foundation
 *
 * @usage
 * Common patterns:
 *
 * Basic title only:
 * ```tsx
 * <PageHeader title="Dashboard" />
 * ```
 *
 * With subtitle and primary action:
 * ```tsx
 * <PageHeader
 *   title="Projects"
 *   subtitle="Your active projects"
 *   actions={[
 *     { label: "New Project", isPrimary: true, onClick: handleCreate }
 *   ]}
 * />
 * ```
 *
 * With search component and multiple actions:
 * ```tsx
 * <PageHeader
 *   title="Team Members"
 *   subtitle={<SearchInput placeholder="Search members..." />}
 *   actions={[
 *     { label: "Invite", isPrimary: true, onClick: handleInvite },
 *     { label: "Export", variant: "outline", onClick: handleExport },
 *     { label: "Settings", href: "/team/settings", variant: "ghost" }
 *   ]}
 * />
 * ```
 *
 * With navigation actions:
 * ```tsx
 * <PageHeader
 *   title="Edit Profile"
 *   actions={[
 *     { label: "Save", isPrimary: true, onClick: handleSave },
 *     { label: "Cancel", href: "/profile", variant: "ghost" }
 *   ]}
 * />
 * ```
 *
 * @example
 * ```tsx
 * <PageHeader
 *   title="Dashboard"
 *   subtitle="Welcome back!"
 *   actions={[
 *     {
 *       label: "New Item",
 *       isPrimary: true,
 *       onClick: () => console.log("clicked")
 *     }
 *   ]}
 *   className="mb-8"
 * />
 * ```
 *
 * @related {@link PageWrapper} - Often used together for page layout
 * @related {@link Button} - Used for rendering actions
 * @composition Uses {@link DropdownMenu} for mobile action menu
 *
 * @cssUtilities
 * - flex-col/flex-row: Responsive layout
 * - gap-6/gap-2: Consistent spacing
 * - text-2xl/text-[30px]: Responsive typography
 * - text-foreground/text-muted-foreground: Text hierarchy
 *
 * @accessibility
 * - Uses semantic h1 for title
 * - Maintains text contrast ratios
 * - Dropdown menu is keyboard navigable
 * - Preserves action button/link semantics
 *
 * @performance
 * - Conditional rendering of subtitle and actions
 * - Mobile-first CSS with responsive modifiers
 * - Efficient action rendering with key prop
 */
export default function PageHeader({
  title,
  subtitle,
  actions = [],
  className
}: PageHeaderProps) {
  // Separate primary actions from secondary actions
  const primaryActions = actions.filter((action) => action.isPrimary)
  const secondaryActions = actions.filter((action) => !action.isPrimary)

  // Render an action (either as button or link)
  const renderAction = (action: PageAction, key: string) => {
    const variant = action.variant || (action.isPrimary ? 'default' : 'outline')

    if (action.href) {
      return (
        <Button key={key} variant={variant} asChild>
          <Link href={action.href}>{action.label}</Link>
        </Button>
      )
    }

    return (
      <Button key={key} variant={variant} onClick={action.onClick}>
        {action.label}
      </Button>
    )
  }

  return (
    <div className={cn('w-full max-w-[1232px] mx-auto', className)}>
      <div className="flex flex-col md:flex-row md:justify-between md:items-center gap-6 md:gap-2">
        <div className="space-y-2">
          <h1 className="text-2xl md:text-[30px] font-bold leading-8 md:leading-9 text-foreground">
            {title}
          </h1>
          {subtitle && (
            <div className="mt-2">
              {typeof subtitle === 'string' ? (
                <p className="text-sm md:text-base text-muted-foreground leading-5 md:leading-6">
                  {subtitle}
                </p>
              ) : (
                subtitle
              )}
            </div>
          )}
        </div>

        {actions.length > 0 && (
          <>
            {/* Desktop buttons */}
            <div className="hidden md:flex items-center gap-2">
              {actions.map((action, index) =>
                renderAction(action, `desktop-${index}`)
              )}
            </div>

            {/* Mobile buttons */}
            <div className="flex md:hidden items-center gap-2">
              {primaryActions.map((action, index) =>
                renderAction(action, `mobile-primary-${index}`)
              )}

              {secondaryActions.length > 0 && (
                <DropdownMenu>
                  <DropdownMenuTrigger asChild>
                    <Button variant="outline" size="icon">
                      <MoreVertical className="h-4 w-4" />
                    </Button>
                  </DropdownMenuTrigger>
                  <DropdownMenuContent align="end">
                    {secondaryActions.map((action) =>
                      action.href ? (
                        <DropdownMenuItem asChild key={action.label}>
                          <Link href={action.href}>{action.label}</Link>
                        </DropdownMenuItem>
                      ) : (
                        <DropdownMenuItem
                          onClick={action.onClick}
                          key={action.label}
                        >
                          {action.label}
                        </DropdownMenuItem>
                      )
                    )}
                  </DropdownMenuContent>
                </DropdownMenu>
              )}
            </div>
          </>
        )}
      </div>
    </div>
  )
}
