import { cn } from '@workspace/ui/lib/utils'
import type { ReactNode } from 'react'
import {
  type PageAction,
  PageHeader,
  type PageHeaderProps
} from '../page-header'

/**
 * Props for the PageWrapper component
 * @remarks
 * Configuration interface for PageWrapper, a layout component that provides:
 * - Optional header with title, subtitle, and actions
 * - Flexible content area with two layout modes
 * - Responsive padding and spacing
 * - Customizable styling
 *
 * @see {@link PageWrapper} for the component implementation
 */
export interface PageWrapperProps {
  /**
   * Header configuration for the page
   * @remarks
   * Configures the page header section with:
   * - Required title: Main heading text
   * - Optional subtitle: Text or custom component below title
   * - Optional actions: Array of clickable/linkable buttons
   *   • label: Button text
   *   • variant: Visual style ('default' | 'destructive' | 'outline' | 'secondary' | 'ghost' | 'link')
   *   • onClick/href: Action handler (mutually exclusive)
   *   • isPrimary: Gives visual emphasis and affects mobile layout
   * - Optional className: Custom styling for header
   *
   * @example
   * ```tsx
   * header={{
   *   title: "Page Title",
   *   subtitle: "Optional description or component",
   *   actions: [
   *     {
   *       label: "Primary Action",
   *       isPrimary: true,
   *       onClick: () => console.log("clicked")
   *     },
   *     {
   *       label: "Secondary Action",
   *       href: "/some-path",
   *       variant: "outline"
   *     }
   *   ]
   * }}
   * ```
   *
   * @see {@link PageHeaderProps} for complete header configuration
   * @see {@link PageAction} for detailed action button options
   */
  header?: PageHeaderProps

  /**
   * Main content for the page
   * @remarks
   * - Rendered in the main content area below header
   * - In 'default' layout: Single column with max-width
   * - In 'bento' layout: 3-column grid on desktop, single column on mobile
   *
   * @example
   * ```tsx
   * // Single column content
   * <div className="space-y-4">
   *   <Card>Content block 1</Card>
   *   <Card>Content block 2</Card>
   * </div>
   *
   * // Bento grid content
   * <>
   *   <Card className="md:col-span-2">Wide card</Card>
   *   <Card>Sidebar card</Card>
   * </>
   * ```
   */
  children: ReactNode

  /**
   * Layout style for the page
   * @remarks
   * - 'default': Single-column layout with max-width (4xl)
   * - 'bento': Responsive grid layout
   *   • Desktop: 3-column grid with 1232px max width
   *   • Mobile: Single column
   * @defaultValue "default"
   */
  layout?: 'default' | 'bento'

  /**
   * Optional CSS classes for the wrapper
   * @remarks
   * - Applied to the wrapper's root container
   * - Combined with default classes using the cn utility
   * - Default classes: 'flex flex-col h-full'
   *
   * @example
   * ```tsx
   * // Custom background
   * className="bg-muted"
   *
   * // Custom padding
   * className="p-8"
   *
   * // Full height with scrolling content
   * className="h-screen overflow-auto"
   * ```
   */
  className?: string
}

/**
 * A flexible page layout component that provides consistent structure and styling.
 *
 * @description
 * PageWrapper is a container component that provides a consistent layout structure
 * for page content. It supports an optional header section and two layout modes:
 * - default: Single-column layout with max-width
 * - bento: Grid-based layout with multiple sections
 *
 * @keywords page-layout, page-container, header-layout, responsive-grid, bento-grid, foundation-component
 * @category Layout
 * @scope Foundation
 *
 * @usage
 * Common patterns:
 *
 * Basic page with title only:
 * ```tsx
 * <PageWrapper header={{ title: "My Page" }}>
 *   <div>Simple content</div>
 * </PageWrapper>
 * ```
 *
 * Dashboard section with actions:
 * ```tsx
 * <PageWrapper
 *   header={{
 *     title: "Dashboard",
 *     subtitle: "Your personal overview",
 *     actions: [
 *       { label: "New Item", isPrimary: true, onClick: () => {} },
 *       { label: "Filter", variant: "outline", onClick: () => {} }
 *     ]
 *   }}
 *   layout="bento"
 * >
 *   <Card className="md:col-span-2">
 *     <MetricsOverview />
 *   </Card>
 *   <Card>
 *     <RecentActivity />
 *   </Card>
 * </PageWrapper>
 * ```
 *
 * Form page with navigation:
 * ```tsx
 * <PageWrapper
 *   header={{
 *     title: "Edit Profile",
 *     actions: [
 *       { label: "Save", isPrimary: true, onClick: handleSave },
 *       { label: "Cancel", href: "/dashboard", variant: "ghost" }
 *     ]
 *   }}
 * >
 *   <Form>
 *     <FormField {...fieldProps} />
 *   </Form>
 * </PageWrapper>
 * ```
 *
 * Settings page with sections:
 * ```tsx
 * <PageWrapper
 *   header={{ title: "Settings" }}
 *   layout="bento"
 * >
 *   <Card className="md:col-span-2">
 *     <h2>General Settings</h2>
 *     <SettingsForm />
 *   </Card>
 *   <Card>
 *     <h2>Quick Actions</h2>
 *     <QuickActions />
 *   </Card>
 * </PageWrapper>
 * ```
 *
 * @example
 * ```tsx
 * // Basic usage
 * <PageWrapper>
 *   <div>Page content</div>
 * </PageWrapper>
 *
 * // With header and custom layout
 * <PageWrapper
 *   header={{
 *     title: "Page Title",
 *     subtitle: "Optional subtitle",
 *     actions: [{
 *       label: "Action",
 *       onClick: () => console.log("clicked")
 *     }]
 *   }}
 *   layout="bento"
 * >
 *   <div>Grid-based content</div>
 * </PageWrapper>
 * ```
 *
 * @param props - Component props
 * @param props.header - Optional header configuration for the page. See {@link PageHeaderProps} for full details
 * @param props.children - Main content to be rendered within the wrapper
 * @param props.layout - Layout style for content organization ('default' | 'bento')
 * @param props.className - Additional CSS classes for the root container
 *
 * @returns A structured page layout with optional header and content areas
 *
 * @related {@link PageHeader} - Used internally for header rendering
 * @related {@link NavigationWrapper} - Often used as parent component
 * @composition Uses {@link cn} for class name merging
 *
 * @cssUtilities
 * - flex-col: Column layout
 * - h-full: Full height
 * - max-w-4xl: Maximum width for default layout
 * - grid-cols-1: Single column on mobile
 * - md:grid-cols-3: Three columns on desktop
 *
 * @accessibility
 * - Maintains proper heading hierarchy with h1 in header
 * - Preserves content structure for screen readers
 * - Supports keyboard navigation through action buttons
 *
 * @performance
 * - Minimal DOM nesting
 * - Conditional rendering of header
 * - Uses utility classes for styling
 */
export default function PageWrapper({
  header,
  children,
  layout = 'default',
  className
}: PageWrapperProps) {
  return (
    <div className={cn('flex flex-col h-full', className)}>
      {header && (
        <div className="py-6 md:py-8 px-4 md:px-6 border-b">
          <PageHeader {...header} />
        </div>
      )}
      <div className={cn('flex-1 px-4 md:px-6 py-6 md:py-8')}>
        <div
          className={cn(
            'mx-auto',
            layout === 'default' ? 'max-w-4xl' : 'w-full max-w-[1232px]',
            layout === 'bento' && 'grid grid-cols-1 md:grid-cols-3 gap-6'
          )}
        >
          {children}
        </div>
      </div>
    </div>
  )
}
