import { Button } from '@workspace/ui/components/button'
import type React from 'react'
import { useEffect, useRef, useState } from 'react'
import type { Project, ProjectSearchBarProps } from './types'

/**
 * A search bar component that allows the user to search for projects.
 * This is a simplified version without external dependencies.
 *
 * @param {ProjectSearchBarProps} props - The props for the component.
 * @returns {React.ReactElement} A div element containing the search bar and project list.
 */
export const ProjectSearchBar: React.FC<ProjectSearchBarProps> = ({
  onChange,
  placeholder = 'Search projects...'
}) => {
  const [searchTerm, setSearchTerm] = useState('')
  const [isOpen, setIsOpen] = useState(false)
  const [items, setItems] = useState<Project[]>([])
  const [selectedIndex, setSelectedIndex] = useState(-1)
  const [debouncedSearchTerm, setDebouncedSearchTerm] = useState('')
  const resultsRef = useRef<HTMLDivElement>(null)

  // Mock data - in real implementation this would come from API
  const mockProjects: Project[] = [
    { id: '1', name: 'Project Alpha', description: 'A test project' },
    { id: '2', name: 'Project Beta', description: 'Another test project' },
    { id: '3', name: 'Project Gamma', description: 'Yet another test project' },
    {
      id: '4',
      name: 'Deploy Frontend',
      description: 'Frontend deployment project'
    },
    { id: '5', name: 'API Service', description: 'Backend API service project' }
  ]

  // Handle debounced search term
  useEffect(() => {
    const handler = setTimeout(() => {
      setDebouncedSearchTerm(searchTerm)
    }, 300)

    return () => {
      clearTimeout(handler)
    }
  }, [searchTerm])

  // Search projects on debounced input change
  useEffect(() => {
    if (debouncedSearchTerm.trim()) {
      const filtered = mockProjects.filter(
        (project) =>
          project.name
            .toLowerCase()
            .includes(debouncedSearchTerm.toLowerCase()) ||
          project.description
            ?.toLowerCase()
            .includes(debouncedSearchTerm.toLowerCase())
      )
      setItems(filtered)
      setIsOpen(filtered.length > 0)
    } else {
      setItems([])
      setIsOpen(false)
    }
  }, [debouncedSearchTerm])

  // Handle keyboard navigation
  const handleKeyDown = (e: React.KeyboardEvent) => {
    if (!isOpen) return

    // Arrow down
    if (e.key === 'ArrowDown') {
      e.preventDefault()
      setSelectedIndex((prev: number) =>
        prev < items.length - 1 ? prev + 1 : prev
      )
    }
    // Arrow up
    else if (e.key === 'ArrowUp') {
      e.preventDefault()
      setSelectedIndex((prev: number) => (prev > 0 ? prev - 1 : 0))
    }
    // Enter
    else if (e.key === 'Enter' && selectedIndex >= 0 && items[selectedIndex]) {
      e.preventDefault()
      handleSelectItem(items[selectedIndex])
    }
    // Escape
    else if (e.key === 'Escape') {
      e.preventDefault()
      setIsOpen(false)
    }
  }

  // Handle item selection
  const handleSelectItem = (project: Project) => {
    if (onChange) {
      onChange(project)
    }
    setSearchTerm(project.name)
    setIsOpen(false)
    setSelectedIndex(-1)
  }

  return (
    <div className="relative w-full lg:w-[459px]">
      {/* Search input */}
      <div className="relative flex w-full">
        <div className="absolute left-2 top-1/2 transform -translate-y-1/2 text-gray-400">
          <svg
            xmlns="http://www.w3.org/2000/svg"
            width="16"
            height="16"
            viewBox="0 0 24 24"
            fill="none"
            stroke="currentColor"
            strokeWidth="2"
            strokeLinecap="round"
            strokeLinejoin="round"
            aria-hidden="true"
          >
            <circle cx="11" cy="11" r="8" />
            <line x1="21" y1="21" x2="16.65" y2="16.65" />
          </svg>
        </div>
        <input
          type="search"
          value={searchTerm}
          onChange={(e) => setSearchTerm(e.target.value)}
          onKeyDown={handleKeyDown}
          placeholder={placeholder}
          className="w-full pl-8 px-3 py-2 border rounded"
          onFocus={() => searchTerm.trim() && setIsOpen(items.length > 0)}
          onBlur={() => setTimeout(() => setIsOpen(false), 200)} // Delay to allow clicking on results
          aria-expanded={isOpen}
          aria-controls={isOpen ? 'project-search-results' : undefined}
          aria-label="Search projects"
        />
      </div>

      {/* Dropdown results */}
      {isOpen && (
        <div
          ref={resultsRef}
          id="project-search-results"
          className="absolute w-full mt-1 bg-white border rounded-md shadow-lg max-h-60 overflow-y-auto z-10"
          tabIndex={-1}
        >
          <div className="px-3 py-2 text-xs text-gray-500 font-medium">
            Suggestions
          </div>
          <ul>
            {items.map((project: Project, index: number) => (
              <Button
                key={project.id}
                variant="ghost"
                className={`w-full justify-start px-3 py-2 h-auto ${
                  selectedIndex === index ? 'bg-gray-100' : ''
                }`}
                onClick={() => handleSelectItem(project)}
                onMouseEnter={() => setSelectedIndex(index)}
                tabIndex={0}
              >
                <div className="flex flex-col items-start text-left">
                  <div className="font-medium">{project.name}</div>
                  {project.description && (
                    <div className="text-sm text-gray-500">
                      {project.description}
                    </div>
                  )}
                </div>
              </Button>
            ))}
          </ul>
          {items.length === 0 && (
            <div className="px-3 py-4 text-center text-gray-500">
              No projects found matching "{debouncedSearchTerm}"
            </div>
          )}
        </div>
      )}
    </div>
  )
}
