'use client'

import { LaconicMark } from '@/components/assets/laconic-mark'
import { UserButton } from '@clerk/nextjs'
import { Button } from '@workspace/ui/components/button'
import {
  Sheet,
  SheetClose,
  SheetContent,
  SheetTitle,
  SheetTrigger
} from '@workspace/ui/components/sheet'
import { CreditCard, Menu, Shapes } from 'lucide-react'
import Link from 'next/link'
import type React from 'react'
import { DarkModeToggle } from '../dark-mode-toggle'
import { NavigationItem } from '../navigation-item'
import type { TopNavigationConfig } from '../types'
import { WalletSessionBadge } from '../wallet-session-badge'

/**
 * Props for the TopNavigation component
 * @remarks
 * Configuration interface for TopNavigation, a layout component that provides:
 * - Responsive navigation bar with mobile drawer
 * - Left and right navigation items
 * - Dark mode toggle
 * - User authentication button
 * - Wallet session badge
 * - Logo/home link
 *
 * @see {@link TopNavigation} for the component implementation
 */
export interface TopNavigationProps {
  /**
   * Configuration for navigation items and layout
   * @remarks
   * - Defines left and right navigation items
   * - Each item can have label, href/onClick, icon, and active state
   * - Items are rendered as buttons or links based on presence of href
   * - Mobile view combines all items into a drawer menu
   *
   * @example
   * ```tsx
   * config={{
   *   leftItems: [
   *     { label: 'Projects', href: '/projects', active: true },
   *     { label: 'Wallet', href: '/wallets', icon: WalletIcon }
   *   ],
   *   rightItems: [
   *     { label: 'Support', href: '/support' },
   *     {
   *       label: 'Documentation',
   *       onClick: () => window.open('https://docs.example.com')
   *     }
   *   ]
   * }}
   * ```
   *
   * @defaultValue
   * ```tsx
   * {
   *   leftItems: [
   *     { label: 'Projects', href: '/projects' },
   *     { label: 'Wallet', href: '/wallets' }
   *   ],
   *   rightItems: [
   *     { label: 'Support', href: '/support' },
   *     { label: 'Documentation', href: '/documentation' }
   *   ]
   * }
   * ```
   */
  config?: TopNavigationConfig

  /**
   * Optional child elements
   * @remarks
   * - Can be used to add custom elements to the navigation
   * - Rendered after the default navigation items
   * - Not commonly used as the config prop handles most use cases
   */
  children?: React.ReactNode
}

/**
 * A responsive navigation bar component with mobile support and integrated features.
 *
 * @description
 * TopNavigation is a foundational component that provides:
 * - Responsive navigation with mobile drawer menu
 * - Configurable left and right navigation items
 * - Integrated dark mode toggle
 * - User authentication button
 * - Wallet session display
 * - Logo/home link
 *
 * @keywords navigation, header, responsive, mobile-menu, foundation-component
 * @category Navigation
 * @scope Foundation
 *
 * @usage
 * Common patterns:
 *
 * Basic navigation:
 * ```tsx
 * <TopNavigation
 *   config={{
 *     leftItems: [
 *       { label: 'Dashboard', href: '/dashboard' },
 *       { label: 'Projects', href: '/projects' }
 *     ],
 *     rightItems: [
 *       { label: 'Settings', href: '/settings' }
 *     ]
 *   }}
 * />
 * ```
 *
 * With active states and icons:
 * ```tsx
 * <TopNavigation
 *   config={{
 *     leftItems: [
 *       {
 *         label: 'Dashboard',
 *         href: '/dashboard',
 *         icon: HomeIcon,
 *         active: true
 *       },
 *       {
 *         label: 'Analytics',
 *         href: '/analytics',
 *         icon: ChartIcon
 *       }
 *     ]
 *   }}
 * />
 * ```
 *
 * With click handlers:
 * ```tsx
 * <TopNavigation
 *   config={{
 *     rightItems: [
 *       {
 *         label: 'Help',
 *         onClick: () => setHelpOpen(true),
 *         icon: HelpIcon
 *       }
 *     ]
 *   }}
 * />
 * ```
 *
 * @example
 * ```tsx
 * // Basic usage
 * <TopNavigation />
 *
 * // Custom navigation items
 * <TopNavigation
 *   config={{
 *     leftItems: [
 *       { label: 'Home', href: '/' },
 *       { label: 'About', href: '/about' }
 *     ]
 *   }}
 * />
 * ```
 *
 * @param props - Component props
 * @param props.config - Navigation configuration object
 * @param props.children - Optional child elements
 *
 * @returns A responsive navigation bar with mobile support
 *
 * @related {@link NavigationItem} - Used for individual nav items
 * @related {@link DarkModeToggle} - Integrated dark mode control
 * @related {@link WalletSessionBadge} - Displays wallet info
 * @composition Uses {@link Sheet} for mobile menu
 *
 * @cssUtilities
 * - sticky: Fixed to top of viewport
 * - z-50: High z-index for overlay
 * - border-b: Bottom border
 * - bg-background: Theme-aware background
 * - text-foreground: Theme-aware text
 *
 * @accessibility
 * - Uses semantic header and nav elements
 * - Includes sr-only labels for screen readers
 * - Supports keyboard navigation
 * - Mobile menu follows drawer pattern
 *
 * @performance
 * - Conditionally renders mobile/desktop views
 * - Uses CSS utilities for styling
 * - Lazy loads mobile drawer content
 */
export default function TopNavigation({
  config = {
    leftItems: [
      { icon: Shapes, label: 'Projects', href: '/projects' },
    ],
    rightItems: [
      { icon: CreditCard, label: 'Purchase', href: '/purchase' },
      { label: 'Support', href: '/support' },
      { label: 'Documentation', href: '/documentation' }
    ]
  }
}: TopNavigationProps) {
  const { leftItems = [], rightItems = [] } = config

  return (
    <header className="sticky top-0 z-50 w-full border-b border-border/40 bg-background text-foreground">
      <div className="container flex h-14 items-center px-4 w-full min-w-full">
        {/* Mobile menu trigger - positioned on the left */}
        <div className="md:hidden mr-2">
          <Sheet>
            <SheetTrigger asChild>
              <Button variant="ghost" size="icon" className="hover:bg-accent">
                <Menu className="h-5 w-5" />
                <span className="sr-only">Toggle menu</span>
              </Button>
            </SheetTrigger>
            <SheetContent
              side="left"
              className="w-[280px] p-0 bg-background border-r border-border"
            >
              <SheetTitle className="sr-only">Navigation Menu</SheetTitle>
              <div className="p-6 border-b border-border flex items-center justify-between">
                <Link href="/" className="flex items-center">
                  <LaconicMark className="h-6 w-6" />
                </Link>
                <SheetClose className="rounded-sm opacity-70 ring-offset-background transition-opacity hover:opacity-100 focus:outline-none focus:ring-2 focus:ring-ring focus:ring-offset-2 disabled:pointer-events-none data-[state=open]:bg-secondary">
                  <span className="sr-only">Close</span>
                </SheetClose>
              </div>
              <div className="flex-1 py-6">
                <nav className="flex flex-col space-y-6">
                  {[...leftItems, ...rightItems].map((item) => (
                    <NavigationItem
                      key={`mobile-${item.label}`}
                      href={item.href}
                      onClick={item.onClick}
                      active={item.active}
                      isDrawerItem={true}
                    >
                      {item.label}
                    </NavigationItem>
                  ))}
                </nav>
              </div>
            </SheetContent>
          </Sheet>
        </div>

        <Link href="/" className="mr-4 flex items-center">
          <LaconicMark className="h-8 w-8" />
        </Link>

        <div className="hidden md:flex items-center space-x-1">
          {leftItems.map((item) => (
            <NavigationItem
              key={`left-${item.label}`}
              href={item.href}
              onClick={item.onClick}
              icon={item.icon}
              active={item.active}
            >
              {item.label}
            </NavigationItem>
          ))}
        </div>

        <div className="flex flex-1 items-center justify-end space-x-4">
          <nav className="hidden md:flex items-center space-x-1">
            {rightItems.map((item) => (
              <NavigationItem
                key={`right-${item.label}`}
                href={item.href}
                onClick={item.onClick}
                icon={item.icon}
                active={item.active}
              >
                {item.label}
              </NavigationItem>
            ))}
          </nav>

          <DarkModeToggle />
          <UserButton />
          <WalletSessionBadge />
        </div>
      </div>
    </header>
  )
}
