'use client'

import { Button } from '@workspace/ui/components/button'
import { cn } from '@workspace/ui/lib/utils'
import type { LucideIcon } from 'lucide-react'
import Link from 'next/link'
import type React from 'react'

/**
 * Props for the NavigationItem component
 * @remarks
 * Configuration interface for NavigationItem, a flexible navigation element that:
 * - Supports both link and button behaviors
 * - Handles mobile drawer and desktop navigation styles
 * - Includes icon support
 * - Provides active state styling
 * - Uses shadcn/ui Button component for consistent styling
 *
 * @see {@link NavigationItem} for the component implementation
 */
export interface NavigationItemProps {
  /**
   * URL for link navigation
   * @remarks
   * - When provided, renders as a Next.js Link
   * - Takes precedence over onClick
   * - Uses Next.js routing for client-side navigation
   *
   * @example
   * ```tsx
   * href="/dashboard"
   * href="/settings/profile"
   * ```
   */
  href?: string

  /**
   * Click handler for button behavior
   * @remarks
   * - Used when href is not provided
   * - Renders as a button element
   * - Useful for actions that don't navigate
   *
   * @example
   * ```tsx
   * onClick={() => setIsOpen(true)}
   * onClick={() => handleLogout()}
   * ```
   */
  onClick?: () => void

  /**
   * Optional CSS classes
   * @remarks
   * - Applied to the root element
   * - Combined with default classes using cn utility
   * - Different defaults for drawer vs regular items
   *
   * @example
   * ```tsx
   * className="text-primary"
   * className="hidden lg:flex"
   * ```
   */
  className?: string

  /**
   * Active state flag
   * @remarks
   * - Adds semibold font weight when true
   * - In drawer mode, also changes text color
   * - Use for current page/section indication
   *
   * @example
   * ```tsx
   * active={pathname === '/dashboard'}
   * active={section === 'settings'}
   * ```
   *
   * @defaultValue false
   */
  active?: boolean

  /**
   * Optional Lucide icon component
   * @remarks
   * - Rendered before children when provided
   * - Only shown in desktop navigation
   * - Sized and spaced automatically
   *
   * @example
   * ```tsx
   * icon={HomeIcon}
   * icon={Settings}
   * ```
   */
  icon?: LucideIcon

  /**
   * Content of the navigation item
   * @remarks
   * - Typically a text label
   * - Can include other elements
   * - Positioned after icon if present
   *
   * @example
   * ```tsx
   * children="Dashboard"
   * children={<>Home <Badge>New</Badge></>}
   * ```
   */
  children: React.ReactNode

  /**
   * Button variant from shadcn/ui
   * @remarks
   * - Only applies to desktop navigation
   * - Drawer items use custom styling
   * - Uses shadcn/ui Button variants
   *
   * @example
   * ```tsx
   * variant="default"
   * variant="ghost"
   * ```
   *
   * @defaultValue "ghost"
   */
  variant?:
    | 'default'
    | 'destructive'
    | 'outline'
    | 'secondary'
    | 'ghost'
    | 'link'

  /**
   * Flag for drawer-specific styling
   * @remarks
   * - When true, uses simpler drawer-specific styles
   * - Affects hover and active states
   * - Changes padding and spacing
   *
   * @defaultValue false
   */
  isDrawerItem?: boolean
}

/**
 * A flexible navigation item component that adapts to mobile and desktop contexts.
 *
 * @description
 * NavigationItem is a foundational component that:
 * - Renders as either a link or button
 * - Adapts styling for mobile drawer or desktop navigation
 * - Supports icons and active states
 * - Maintains consistent styling with shadcn/ui
 *
 * @keywords navigation, link, button, responsive, foundation-component
 * @category Navigation
 * @scope Foundation
 *
 * @usage
 * Common patterns:
 *
 * Basic link:
 * ```tsx
 * <NavigationItem href="/dashboard">
 *   Dashboard
 * </NavigationItem>
 * ```
 *
 * With icon and active state:
 * ```tsx
 * <NavigationItem
 *   href="/settings"
 *   icon={Settings}
 *   active={true}
 * >
 *   Settings
 * </NavigationItem>
 * ```
 *
 * As a button with click handler:
 * ```tsx
 * <NavigationItem
 *   onClick={() => setIsOpen(true)}
 *   variant="ghost"
 * >
 *   Open Menu
 * </NavigationItem>
 * ```
 *
 * In mobile drawer:
 * ```tsx
 * <NavigationItem
 *   href="/profile"
 *   isDrawerItem={true}
 *   active={isCurrentPage}
 * >
 *   Profile
 * </NavigationItem>
 * ```
 *
 * @example
 * ```tsx
 * // Basic usage
 * <NavigationItem href="/home">Home</NavigationItem>
 *
 * // With all props
 * <NavigationItem
 *   href="/settings"
 *   icon={Settings}
 *   active={true}
 *   variant="default"
 *   className="my-custom-class"
 *   isDrawerItem={false}
 * >
 *   Settings
 * </NavigationItem>
 * ```
 *
 * @param props - Component props
 * @param props.href - URL for link navigation
 * @param props.onClick - Click handler for button behavior
 * @param props.className - Additional CSS classes
 * @param props.active - Active state flag
 * @param props.icon - Optional Lucide icon component
 * @param props.children - Content of the navigation item
 * @param props.variant - Button variant from shadcn/ui
 * @param props.isDrawerItem - Flag for drawer-specific styling
 *
 * @returns A navigation item as either a link or button
 *
 * @related {@link TopNavigation} - Parent component
 * @composition Uses {@link Button} from shadcn/ui
 *
 * @cssUtilities
 * Desktop:
 * - font-semibold: Applied when active
 * - mr-2: Icon margin
 * - h-4 w-4: Icon size
 *
 * Drawer:
 * - px-6 py-1: Padding
 * - text-sm: Font size
 * - font-medium: Font weight
 * - hover:text-white/80: Hover state
 *
 * @accessibility
 * - Maintains button/link semantics
 * - Preserves keyboard navigation
 * - Supports screen readers
 * - Indicates current page
 *
 * @performance
 * - Conditional rendering based on props
 * - Uses CSS utilities
 * - Minimal state management
 */
export function NavigationItem({
  href,
  onClick,
  className,
  active = false,
  icon: Icon,
  children,
  variant = 'ghost',
  isDrawerItem = false
}: NavigationItemProps) {
  // For drawer items, use a simpler styling approach
  if (isDrawerItem) {
    const content = (
      <div
        className={cn(
          'px-6 py-1 text-sm font-medium transition-colors hover:text-white/80',
          active && 'text-white font-semibold',
          className
        )}
      >
        {children}
      </div>
    )

    if (href) {
      return <Link href={href}>{content}</Link>
    }

    return <Button onClick={onClick}>{content}</Button>
  }

  // For regular navigation items, use the Button component
  const content = (
    <>
      {Icon && <Icon className="h-4 w-4" />}
      {children}
    </>
  )

  const buttonClassName = cn(active && 'font-semibold', className)

  if (href) {
    return (
      <Button variant={variant} asChild className={buttonClassName}>
        <Link href={href}>{content}</Link>
      </Button>
    )
  }

  return (
    <Button variant={variant} onClick={onClick} className={buttonClassName}>
      {content}
    </Button>
  )
}
