import type { ReactNode } from 'react'
import type { PageAction, PageHeaderProps } from './page-header'

/**
 * Props for the NavigationWrapper component
 * @remarks
 * Container component that wraps page content with navigation functionality.
 * Typically used as the outer container for PageWrapper components.
 */
export type NavigationWrapperProps = {
  /**
   * Content to be displayed within the navigation wrapper
   * @remarks
   * - Typically contains {@link PageWrapper} components
   * - Can include any valid React nodes
   */
  children: ReactNode

  /**
   * Optional CSS class name for custom styling
   * @remarks Applied to the wrapper's root container element
   */
  className?: string
}

// Re-export types from component folders
export type { PageAction, PageHeaderProps }

/**
 * Layout options for page content
 * @remarks

 * - default: Single-column layout
 * - bento: Grid-based layout with multiple sections
 */
export type PageWrapperLayout = 'default' | 'bento'

/**
 * Configuration for the main page container
 * @property {PageHeaderProps} header - Header configuration for the page
 * @property {ReactNode} children - Main content for the page
 * @property {PageWrapperLayout} layout - Layout style for the page
 * @property {string} className - Custom CSS classes for the wrapper
 * @property {string} contentClassName - Custom CSS classes for the content area
 * Main container for page content with optional header, flexible layout options,
 * and customizable styling for both wrapper and content areas.
 */
export interface PageWrapperProps {
  /**
   * Header configuration for the page
   * @see {@link PageHeaderProps}
   */
  header?: PageHeaderProps
  /**
   * Main content for the page
   * @remarks Rendered in the main content area below header
   */
  children: ReactNode
  /**
   * Layout style for the page
   * @see {@link PageWrapperLayout}
   * @defaultValue "default"
   */
  layout?: PageWrapperLayout
  /**
   * Optional CSS classes for the wrapper
   * @remarks Applied to the wrapper's root container
   */
  className?: string
  /**
   * Optional CSS classes for the content area
   * @remarks
   * - Applied to the main content container
   * - Separate from the wrapper's root className
   */
  contentClassName?: string
}
