import { useCallback, useEffect, useState } from 'react'
import { toast } from 'sonner'

/**
 * `useCheckBalance` is a custom React hook that checks if the balance in a wallet iframe is sufficient for a given amount.
 * It communicates with the iframe to request a balance check and updates the state based on the response.
 *
 * @param amount - The amount to check against the wallet balance. This should be a string representation of the amount.
 * @param iframeId - The ID of the iframe element containing the wallet. This ID is used to locate the iframe in the DOM.
 * @returns An object containing:
 *   - `isBalanceSufficient`: A boolean state variable indicating whether the balance is sufficient (true) or not (false). It is initially undefined.
 *   - `checkBalance`: A function that triggers the balance check by sending a message to the wallet iframe.
 *
 * @example
 * ```tsx
 * const { isBalanceSufficient, checkBalance } = useCheckBalance("100", "myWalletIframe");
 *
 * // To trigger the balance check:
 * checkBalance();
 *
 * // To conditionally render based on balance:
 * {isBalanceSufficient === true ? <p>Balance is sufficient!</p> : isBalanceSufficient === false ? <p>Balance is insufficient.</p> : <p>Checking balance...</p>}
 * ```
 *
 * @remarks
 * - The hook uses `useState` to manage the `isBalanceSufficient` state.
 * - The `checkBalance` function uses `useCallback` to prevent unnecessary re-renders.
 * - The hook uses `useEffect` to listen for messages from the wallet iframe and update the `isBalanceSufficient` state accordingly.
 * - It expects the iframe to send a message of type 'IS_SUFFICIENT' with a boolean data field.
 * - It uses `VITE_LACONICD_CHAIN_ID` and `VITE_WALLET_IFRAME_URL` from the environment variables for the chain ID and iframe URL, respectively.
 * - Error logging is included if the iframe is not found or not loaded.
 */
const useCheckBalance = (amount: string, iframeId: string) => {
  const [isBalanceSufficient, setIsBalanceSufficient] = useState<boolean>()

  const iframeUrl = String(process.env.NEXT_PUBLIC_WALLET_IFRAME_URL) || ''
  const chainId = String(process.env.NEXT_PUBLIC_LACONICD_CHAIN_ID) || ''
  
  const checkBalance = useCallback(() => {
    const iframe = document.getElementById(iframeId) as HTMLIFrameElement

    if (!iframe || !iframe.contentWindow) {
      console.error(`Iframe with ID "${iframeId}" not found or not loaded`)
      return
    }

    iframe.contentWindow.postMessage(
      {
        type: 'CHECK_BALANCE',
        chainId,
        amount
      },
      iframeUrl
    )
  }, [iframeId, amount, iframeUrl, chainId])

  useEffect(() => {
    toast.info('Checking balance from useEffect...')
    const handleMessage = (event: MessageEvent) => {
      if (event.origin !== iframeUrl) return

      toast.info('Is Sufficient?', event.data.type)
      if (event.data.type !== 'IS_SUFFICIENT') return

      setIsBalanceSufficient(event.data.data)
    }

    window.addEventListener('message', handleMessage)
    return () => {
      toast.info('Removing event listener from useEffect...')
      window.removeEventListener('message', handleMessage)
    }
  }, [iframeUrl])

  return { isBalanceSufficient, checkBalance }
}

export default useCheckBalance
