import React, { useState } from 'react'

interface GitHubUser {
  login: string
  name?: string
  avatar_url?: string
}

/**
 * Renders a button that allows users to log in or log out with their GitHub account.
 * It displays the user's avatar and username when logged in, and provides a logout option.
 *
 * @returns {React.ReactElement} A dropdown element containing the GitHub session button.
 */
export function GitHubSessionButton() {
  // This would use a hook like useGitHubAuth
  // Using mock data for now as we migrate the component structure
  const [isAuthenticated, setIsAuthenticated] = useState(false)
  const [user, setUser] = useState<GitHubUser | null>(null)

  // Example user data - would come from authentication in real implementation
  const mockUser: GitHubUser = {
    login: 'github-user',
    name: 'GitHub User',
    avatar_url: undefined
  }

  const login = () => {
    console.log('GitHub login')
    setIsAuthenticated(true)
    setUser(mockUser)
  }

  const logout = () => {
    console.log('GitHub logout')
    setIsAuthenticated(false)
    setUser(null)
  }

  return (
    <div className="relative inline-block text-left">
      <div>
        <button
          type="button"
          className={`relative inline-flex items-center justify-center rounded-md border p-2 ${
            isAuthenticated
              ? 'text-green-500 border-green-500'
              : 'text-gray-500 border-gray-300'
          }`}
          id="github-menu-button"
          aria-expanded="false"
          aria-haspopup="true"
        >
          {/* GitHub icon SVG */}
          <svg
            xmlns="http://www.w3.org/2000/svg"
            width="20"
            height="20"
            viewBox="0 0 24 24"
            fill="none"
            stroke="currentColor"
            strokeWidth="2"
            strokeLinecap="round"
            strokeLinejoin="round"
            aria-hidden="true"
          >
            <path d="M9 19c-5 1.5-5-2.5-7-3m14 6v-3.87a3.37 3.37 0 0 0-.94-2.61c3.14-.35 6.44-1.54 6.44-7A5.44 5.44 0 0 0 20 4.77 5.07 5.07 0 0 0 19.91 1S18.73.65 16 2.48a13.38 13.38 0 0 0-7 0C6.27.65 5.09 1 5.09 1A5.07 5.07 0 0 0 5 4.77a5.44 5.44 0 0 0-1.5 3.78c0 5.42 3.3 6.61 6.44 7A3.37 3.37 0 0 0 9 18.13V22" />
          </svg>
          {isAuthenticated && (
            <span className="absolute top-0 right-0 block h-2 w-2 rounded-full bg-green-500" />
          )}
        </button>
      </div>

      {/* Dropdown menu - would be shown/hidden with state */}
      <div
        className="hidden absolute right-0 z-10 mt-2 w-56 origin-top-right rounded-md bg-white shadow-lg ring-1 ring-black ring-opacity-5 focus:outline-none"
        role="menu"
        aria-orientation="vertical"
        aria-labelledby="github-menu-button"
        tabIndex={-1}
      >
        {isAuthenticated && user ? (
          <div className="py-1">
            <div className="flex items-center gap-2 p-2">
              <div className="h-8 w-8 rounded-full bg-gray-200 flex items-center justify-center overflow-hidden">
                {user.avatar_url ? (
                  <img
                    src={user.avatar_url}
                    alt={user.login}
                    className="h-full w-full object-cover"
                  />
                ) : (
                  <span className="text-sm font-medium">
                    {/* Fallback: first two letters of username */}
                    {user.login.slice(0, 2).toUpperCase() || 'GH'}
                  </span>
                )}
              </div>
              <div className="flex flex-col">
                <span className="text-sm font-medium">
                  {user.name || user.login || 'GitHub User'}
                </span>
                <span className="text-xs text-gray-500">
                  {user.login || '@github-user'}
                </span>
              </div>
            </div>
            <button
              type="button"
              className="block w-full text-left px-4 py-2 text-sm text-gray-700 hover:bg-gray-100"
              role="menuitem"
              tabIndex={-1}
              onClick={logout}
            >
              Log out
            </button>
          </div>
        ) : (
          <button
            type="button"
            className="block w-full text-left px-4 py-2 text-sm text-gray-700 hover:bg-gray-100"
            role="menuitem"
            tabIndex={-1}
            onClick={login}
          >
            Log in with GitHub
          </button>
        )}
      </div>
    </div>
  )
}
