import type React from 'react'

/**
 * Props for the StepHeader component
 * @interface StepHeaderProps
 * @property {React.ReactNode} icon - The icon to display next to the header
 * @property {string} title - The title of the current step
 * @property {string} description - A brief description of the current step
 */
interface StepHeaderProps {
  icon: React.ReactNode
  title: string
  description: string
}

/**
 * Displays the header for the current onboarding step
 * Used at the top of each step's content area
 *
 * @component
 * @example
 * ```tsx
 * <StepHeader
 *   icon={<GitPullRequest />}
 *   title="Connect"
 *   description="Connect and import a GitHub repository"
 * />
 * ```
 *
 * @param {StepHeaderProps} props - Component props
 * @returns {JSX.Element} A header with icon, title, and description
 */
export function StepHeader({ icon, title, description }: StepHeaderProps) {
  return (
    <div className="flex items-center gap-3.5 mb-8">
      <div className="flex items-center justify-center w-11 h-11 bg-foreground rounded-lg p-2.5">
        <div className="text-accent">{icon}</div>
      </div>
      <div className="flex flex-col gap-1">
        <h1 className="text-2xl font-semibold text-foreground">{title}</h1>
        <p className="text-sm text-muted-foreground">{description}</p>
      </div>
    </div>
  )
}
