import { Button } from '@workspace/ui/components/button'
import { cn } from '@workspace/ui/lib/utils'
import type { Step } from '../types'

/**
 * Props for the StepNavigation component
 * @interface StepNavigationProps
 * @property {Step} currentStep - The current active step in the onboarding flow
 * @property {() => void} [onPrevious] - Callback function for the previous button
 * @property {() => void} [onNext] - Callback function for the next button
 * @property {string} [nextLabel] - Custom label for the next button
 */
interface StepNavigationProps {
  currentStep: Step
  onPrevious?: () => void
  onNext?: () => void
  nextLabel?: string
}

/** Order of steps in the onboarding flow */
const STEP_ORDER: Step[] = ['connect', 'configure', 'deploy']

/**
 * Navigation component for moving between onboarding steps
 * Displays progress indicators and previous/next buttons
 *
 * @component
 * @example
 * ```tsx
 * <StepNavigation
 *   currentStep="connect"
 *   onPrevious={handlePrevious}
 *   onNext={handleNext}
 *   nextLabel="Continue"
 * />
 * ```
 *
 * @param {StepNavigationProps} props - Component props
 * @returns {JSX.Element} Navigation controls with progress indicators
 */
export function StepNavigation({
  currentStep,
  onPrevious,
  onNext,
  nextLabel = 'Next'
}: StepNavigationProps) {
  const currentStepIndex = STEP_ORDER.indexOf(currentStep)
  const totalSteps = STEP_ORDER.length

  return (
    <div className="flex items-center justify-between mt-8">
      <Button
        variant="ghost"
        onClick={onPrevious}
        disabled={currentStepIndex === 0}
        className={cn(
          'text-foreground hover:text-foreground/80',
          currentStepIndex === 0 &&
            'text-muted-foreground hover:text-muted-foreground'
        )}
      >
        Previous
      </Button>
      <div className="flex gap-2">
        {STEP_ORDER.map((step, index) => (
          <div
            key={step}
            className={cn(
              'h-1 w-8 rounded-full transition-colors',
              index === currentStepIndex ? 'bg-zinc-800' : 'bg-zinc-300'
            )}
          />
        ))}
      </div>
      <Button onClick={onNext} disabled={currentStepIndex === totalSteps - 1}>
        {currentStepIndex === totalSteps - 1 ? 'Deploy' : nextLabel}
      </Button>
    </div>
  )
}
