/**
 * @component SidebarNav
 * @description Sidebar navigation component showing all steps and their status
 *
 * @see https://www.figma.com/design/cfMOy1RJasIu3QyzAMBFxB/Laconic?node-id=571-3500&m=dev
 */

import { Separator } from '@radix-ui/react-dropdown-menu'
import { cn } from '@workspace/ui/lib/utils'
import { FileCog, GitPullRequest, SquareArrowOutDownRight } from 'lucide-react'
import type React from 'react'
import { BackgroundSVG, LaconicIconLettering } from '../common'
import type { Step } from '../types'

/**
 * Props for the StepItem subcomponent
 * @interface StepItemProps
 * @property {React.ReactNode} icon - Icon representing the step
 * @property {string} title - Step title
 * @property {string} description - Step description
 * @property {boolean} isActive - Whether this step is currently active
 */
interface StepItemProps {
  icon: React.ReactNode
  title: string
  description: string
  isActive: boolean
}

/**
 * Individual step item in the sidebar navigation
 * @component
 * @param {StepItemProps} props - Component props
 */
const StepItem = ({ icon, title, description, isActive }: StepItemProps) => (
  <div className="flex items-center gap-3.5">
    <div
      className={cn(
        'flex items-center justify-center w-11 h-11 rounded-lg p-2.5 transition-colors',
        isActive ? 'bg-foreground' : 'bg-primary-foreground border'
      )}
    >
      <div
        className={cn(
          'w-6 h-6 stroke-2 transition-colors',
          isActive ? 'text-accent' : 'text-muted-foreground'
        )}
      >
        {icon}
      </div>
    </div>
    <div className="flex flex-col gap-1">
      <h3
        className={cn(
          'text-lg font-semibold leading-7 transition-colors',
          isActive ? 'text-foreground' : 'text-muted-foreground'
        )}
      >
        {title}
      </h3>
      <p
        className={cn(
          'text-sm leading-5 transition-colors',
          isActive ? 'text-foreground' : 'text-muted-foreground'
        )}
      >
        {description}
      </p>
    </div>
  </div>
)

/**
 * Props for the SidebarNav component
 * @interface SidebarNavProps
 * @property {Step} currentStep - Currently active step in the onboarding flow
 */
interface SidebarNavProps {
  currentStep: Step
}

/**
 * Sidebar navigation component showing all steps and their status
 *
 * @component
 * @example
 * ```tsx
 * <SidebarNav currentStep="connect" />
 * ```
 *
 * Interacts with:
 * - useOnboarding store for step state
 * - StepItem subcomponent for individual step display
 *
 * @param {SidebarNavProps} props - Component props
 * @returns {JSX.Element} Sidebar with step navigation
 */
export function SidebarNav({ currentStep }: SidebarNavProps) {
  return (
    <div className="w-[379px] p-6 bg-primary-foreground rounded-lg shadow-[0_1px_2px_0_rgba(0,0,0,0.06),0_1px_3px_0_rgba(0,0,0,0.1)] relative overflow-hidden">
      <div className="h-5 mb-6 text-zinc-800 dark:text-white">
        <LaconicIconLettering />
      </div>

      <Separator className="mb-6" />

      <div className="space-y-6">
        <StepItem
          icon={<GitPullRequest />}
          title="Connect"
          description="Connect and import a GitHub repo"
          isActive={currentStep === 'connect'}
        />

        <StepItem
          icon={<FileCog />}
          title="Configure"
          description="Define the deployment type"
          isActive={currentStep === 'configure'}
        />

        <StepItem
          icon={<SquareArrowOutDownRight />}
          title="Deploy"
          description="Review and confirm deployment"
          isActive={currentStep === 'deploy'}
        />
      </div>

      <div className="absolute bottom-0 left-0">
        <BackgroundSVG />
      </div>
    </div>
  )
}
