'use client'

import { useEffect, useState } from 'react'
import { useRouter, useParams } from 'next/navigation'
import { useTheme } from 'next-themes'
import { CheckCircle } from 'lucide-react'
import { Button } from '@workspace/ui/components/button'
import { useOnboarding } from '@/components/onboarding/useOnboarding'

export function SuccessStep() {
  const router = useRouter()
  const params = useParams()
  const { resolvedTheme } = useTheme()
  const [mounted, setMounted] = useState(false)
  const providerParam = params?.provider ? String(params.provider) : 'github'
  const { formData, resetOnboarding } = useOnboarding()
  
  // Handle hydration mismatch by waiting for mount
  useEffect(() => {
    setMounted(true)
  }, [])
  
  // Get deployment info from form data
  const repoName = formData.githubRepo ? formData.githubRepo.split('/').pop() : (formData.projectName || 'project')
  const deploymentUrl = formData.deploymentUrl || `https://${repoName}.laconic.deploy`
  const projectId = formData.projectId || 'unknown-id'
  
  // Handle "View Project" button - navigates to project page
  const handleViewProject = () => {
    console.log('Navigating to project with ID:', projectId)
    resetOnboarding() // Reset state for next time
    
    // Navigate to the project detail page using the GraphQL project ID
    router.push(`/projects/${providerParam}/ps/${projectId}`)
  }
  
  // Auto-redirect after a delay (optional)
  useEffect(() => {
    if (mounted && projectId && projectId !== 'unknown-id') {
      const timer = setTimeout(() => {
        handleViewProject()
      }, 3000) // Auto-redirect after 3 seconds
      
      return () => clearTimeout(timer)
    }
  }, [mounted, projectId])
  
  // Don't render UI until after mount to prevent hydration mismatch
  if (!mounted) {
    return null
  }
  
  // Determine if dark mode is active
  const isDarkMode = resolvedTheme === 'dark'
  
  return (
    <div className="w-full h-full flex flex-col items-center justify-center p-8">
      <div className="max-w-md w-full mx-auto">
        {/* Success icon */}
        <div className="mx-auto mb-6 flex justify-center">
          <CheckCircle className="h-16 w-16 text-green-500" />
        </div>
        
        {/* Success header */}
        <h2 className={`text-2xl font-medium ${isDarkMode ? "text-white" : "text-zinc-900"} text-center mb-2`}>
          Successfully Deployed!
        </h2>
        <p className="text-center text-zinc-500 mb-8">
          Your project has been deployed successfully
        </p>
        
        {/* Deployment summary */}
        <div className={`border rounded-md p-4 mb-6 ${isDarkMode ? "border-zinc-700" : "border-zinc-300"}`}>
          <div className="space-y-2">
            <div className="flex justify-between">
              <span className="text-muted-foreground">Project:</span>
              <span className="font-medium">{repoName}</span>
            </div>
            <div className="flex justify-between">
              <span className="text-muted-foreground">URL:</span>
              <span className="font-mono text-xs">{deploymentUrl}</span>
            </div>
            <div className="flex justify-between">
              <span className="text-muted-foreground">Project ID:</span>
              <span className="font-mono text-xs">{projectId}</span>
            </div>
          </div>
        </div>
        
        {/* Next steps section */}
        <div className="mb-8">
          <h3 className={`text-lg font-medium ${isDarkMode ? "text-white" : "text-zinc-900"} mb-4`}>Next steps</h3>
          
          <div className={`border rounded-md overflow-hidden mb-4 ${isDarkMode ? "border-zinc-700" : "border-zinc-300"}`}>
            <div className="flex items-center p-4 justify-between">
              <div>
                <div className={isDarkMode ? "text-white font-medium" : "text-zinc-900 font-medium"}>Setup Domain</div>
                <div className="text-zinc-500 text-sm">Add a custom domain to your project.</div>
              </div>
              <Button variant="outline" className={`rounded-full p-1 w-8 h-8 flex items-center justify-center ${isDarkMode ? "border-zinc-700" : "border-zinc-300"}`}>
                <svg width="16" height="16" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg" className={isDarkMode ? "text-white" : "text-zinc-900"}>
                  <path d="M9 18L15 12L9 6" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round"/>
                </svg>
              </Button>
            </div>
          </div>
        </div>
        
        {/* Action buttons */}
        <div className="flex flex-col space-y-3">
          <Button 
            className="w-full bg-white hover:bg-white/90 text-black flex items-center justify-center"
            onClick={handleViewProject}
            disabled={!projectId || projectId === 'unknown-id'}
          >
            View Project 
            <svg className="ml-2 h-4 w-4" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
              <path d="M5 12H19M19 12L13 6M19 12L13 18" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round"/>
            </svg>
          </Button>
          
          {/* Manual navigation button if auto-redirect fails */}
          {projectId === 'unknown-id' && (
            <p className="text-center text-xs text-muted-foreground">
              Project ID not found. Please navigate to projects manually.
            </p>
          )}
        </div>
      </div>
    </div>
  )
}