/**
 * Shared type definitions for the onboarding flow
 *
 * @see https://www.figma.com/design/cfMOy1RJasIu3QyzAMBFxB/Laconic?node-id=571-3500&m=dev
 */

/**
 * Available steps in the onboarding flow
 */
export type Step = 'connect' | 'configure' | 'deploy'

/**
 * Form data collected during the onboarding process
 * @interface OnboardingFormData
 * @property {string} projectName - Project name
 * @property {string} repoName - Repository name
 * @property {string} repoDescription - Repository description
 * @property {string} framework - Framework used for the project
 * @property {string} access - Access level of the repository
 * @property {string} organizationSlug - Organization slug
 */
export interface OnboardingFormData {
  projectName: string
  repoName: string
  repoDescription: string
  framework: string
  access: 'public' | 'private'
  organizationSlug: string
}

/**
 * GitHub repository information
 * @interface Repository
 * @property {string} id - Unique identifier for the repository
 * @property {string} name - Repository name
 * @property {string} fullName - Full repository name including owner
 * @property {string} [description] - Repository description
 * @property {boolean} [isPrivate] - Whether the repository is private
 * @property {string} [url] - Repository URL
 */
export interface Repository {
  id: string
  name: string
  fullName: string
  description?: string
  isPrivate?: boolean
  url?: string
}

/**
 * Template information for new projects
 * @interface Template
 * @property {string} id - Unique identifier for the template
 * @property {string} name - Template name
 * @property {string} [description] - Template description
 * @property {string} [thumbnail] - Template thumbnail URL
 */
export interface Template {
  id: string
  name: string
  description?: string
  thumbnail?: string
}

/**
 * Deployment type information
 * @interface DeploymentType
 * @property {string} id - Unique identifier for the deployment type
 * @property {string} name - Deployment type name
 * @property {string} [description] - Deployment type description
 */
export interface DeploymentType {
  id: string
  name: string
  description?: string
}

/**
 * Environment variable definition
 * @interface EnvironmentVariable
 * @property {string} key - Environment variable key
 * @property {string} value - Environment variable value
 * @property {boolean} [isSecret] - Whether the variable is a secret
 */
export interface EnvironmentVariable {
  key: string
  value: string
  isSecret?: boolean
}
