'use client'

import { create } from 'zustand'
import { persist } from 'zustand/middleware'

export type Step = 'connect' | 'configure' | 'deploy' | 'success'

export interface EnvironmentVariables {
  key: string
  value: string
}

export interface OnboardingFormData {
  // Connect step
  githubRepo?: string
  
  // Configure step
  deploymentType?: 'auction' | 'lrn'
  deployerCount?: string
  maxPrice?: string
  selectedLrn?: string
  environments?: {
    production: boolean
    preview: boolean
    development: boolean
  }
  environmentVariables?: Record<string, string>
  
  // Deploy step
  deploymentId?: string
  deploymentUrl?: string
  
  // Success step
  projectId?: string
}

interface OnboardingState {
  currentStep: Step
  formData: OnboardingFormData
  setCurrentStep: (step: Step) => void
  setFormData: (data: Partial<OnboardingFormData>) => void
  nextStep: () => void
  previousStep: () => void
  resetOnboarding: () => void
}

const STEP_ORDER: Step[] = ['connect', 'configure', 'deploy', 'success']

export const useOnboarding = create<OnboardingState>()(
  persist(
    (set) => ({
      currentStep: 'connect',
      formData: {},
      
      setCurrentStep: (step) => set({ currentStep: step }),
      
      setFormData: (data) =>
        set((state) => ({
          formData: { ...state.formData, ...data }
        })),
        
      nextStep: () =>
        set((state) => {
          const currentIndex = STEP_ORDER.indexOf(state.currentStep)
          const nextStep = STEP_ORDER[currentIndex + 1]
          return nextStep ? { currentStep: nextStep } : state
        }),
        
      previousStep: () =>
        set((state) => {
          const currentIndex = STEP_ORDER.indexOf(state.currentStep)
          const previousStep = STEP_ORDER[currentIndex - 1]
          return previousStep ? { currentStep: previousStep } : state
        }),
        
      resetOnboarding: () =>
        set({
          currentStep: 'connect',
          formData: {}
        })
    }),
    {
      name: 'laconic-onboarding-storage'
    }
  )
)