// FixedProjectCard.tsx - With fixed navigation
'use client';

import { useRouter } from 'next/navigation'
import React, { type ComponentPropsWithoutRef, useCallback } from 'react'
import { Card, CardContent, CardHeader } from '@workspace/ui/components/card'
import { AlertTriangle, GitBranch, MoreVertical } from 'lucide-react'
import { getInitials } from '@/utils/getInitials'
import {
  Avatar,
  AvatarFallback,
  AvatarImage
} from '@workspace/ui/components/avatar'

/**
 * Status types for project deployment status
 */
export type ProjectStatus = 'success' | 'in-progress' | 'failure' | 'pending'

/**
 * Props for the ProjectCard component
 */
interface ProjectCardProps extends ComponentPropsWithoutRef<'div'> {
  project: any
  status?: ProjectStatus
}

/**
 * ProjectStatusDot component
 */
const ProjectStatusDot = ({ status }: { status: ProjectStatus }) => {
  const statusColors = {
    success: 'bg-green-500',
    'in-progress': 'bg-yellow-500',
    failure: 'bg-red-500',
    pending: 'bg-gray-400'
  };

  return <div className={`h-2 w-2 rounded-full ${statusColors[status]}`} />;
};

/**
 * ProjectDeploymentInfo component
 */
const ProjectDeploymentInfo = ({ project }: { project: any }) => {
  // Get formatted time with robust error handling
  const getTimeDisplay = (dateStr?: string | number | Date): string => {
    if (!dateStr) return 'No date';
    
    try {
      // Handle different input types
      let date: Date;
      
      if (dateStr instanceof Date) {
        date = dateStr;
      } else if (typeof dateStr === 'number') {
        date = new Date(dateStr);
      } else if (typeof dateStr === 'string') {
        // Handle various string formats
        date = new Date(dateStr);
        
        // If the string date is invalid, try parsing as timestamp
        if (isNaN(date.getTime())) {
          const timestamp = parseInt(dateStr, 10);
          if (!isNaN(timestamp)) {
            date = new Date(timestamp);
          }
        }
      } else {
        return 'Invalid date';
      }
      
      // Check if date is valid
      if (isNaN(date.getTime())) {
        console.warn('Invalid date received:', dateStr);
        return 'Invalid date';
      }
      
      const now = new Date();
      const diffInMs = now.getTime() - date.getTime();
      
      // Handle future dates
      if (diffInMs < 0) {
        return 'Future date';
      }
      
      const diffInMinutes = Math.floor(diffInMs / (1000 * 60));
      const diffInHours = Math.floor(diffInMs / (1000 * 60 * 60));
      const diffInDays = Math.floor(diffInMs / (1000 * 60 * 60 * 24));
      
      if (diffInMinutes < 1) {
        return 'Just now';
      } else if (diffInMinutes < 60) {
        return `${diffInMinutes} minute${diffInMinutes === 1 ? '' : 's'} ago`;
      } else if (diffInHours < 24) {
        return `${diffInHours} hour${diffInHours === 1 ? '' : 's'} ago`;
      } else if (diffInDays < 30) {
        return `${diffInDays} day${diffInDays === 1 ? '' : 's'} ago`;
      } else {
        // For dates older than 30 days, show the actual date
        return date.toLocaleDateString();
      }
    } catch (error) {
      console.error('Error parsing date:', dateStr, error);
      return 'Invalid date';
    }
  };

  const createdAt = project.deployments?.[0]?.createdAt;
  const branch = project.deployments?.[0]?.branch || 'main';
  
  return (
    <div className="flex items-center text-sm text-muted-foreground">
      <span>{getTimeDisplay(createdAt)}</span>
      <span className="mx-1">on</span>
      <GitBranch className="w-4 h-4 mr-1" />
      <span>{branch}</span>
    </div>
  );
};

/**
 * ProjectCardActions component
 */
const ProjectCardActions = ({ 
  onSettingsClick, 
  onDeleteClick 
}: { 
  onSettingsClick: (e: React.MouseEvent) => void, 
  onDeleteClick: (e: React.MouseEvent) => void 
}) => {
  const [menuOpen, setMenuOpen] = React.useState(false);
  
  const toggleMenu = (e: React.MouseEvent) => {
    e.stopPropagation();
    e.preventDefault();
    setMenuOpen(!menuOpen);
  };

  // Close menu when clicking outside
  React.useEffect(() => {
    const handleClickOutside = () => setMenuOpen(false);
    if (menuOpen) {
      document.addEventListener('click', handleClickOutside);
      return () => document.removeEventListener('click', handleClickOutside);
    }
  }, [menuOpen]);

  return (
    <div className="relative">
      <button 
        onClick={toggleMenu}
        className="flex h-8 w-8 items-center justify-center rounded-md border border-input bg-background text-muted-foreground hover:bg-accent hover:text-accent-foreground"
      >
        <MoreVertical className="h-4 w-4" />
      </button>
      
      {menuOpen && (
        <div className="absolute right-0 top-full mt-2 w-48 rounded-md border bg-background shadow-lg z-10">
          <div className="py-1">
            <button
              className="w-full px-4 py-2 text-sm text-left hover:bg-accent"
              onClick={onSettingsClick}
            >
              Settings
            </button>
            <button
              className="w-full px-4 py-2 text-sm text-left text-destructive hover:bg-accent"
              onClick={onDeleteClick}
            >
              Delete
            </button>
          </div>
        </div>
      )}
    </div>
  );
};

/**
 * FixedProjectCard component
 */
export const FixedProjectCard = ({
  className,
  project,
  status = 'failure',
  ...props
}: ProjectCardProps) => {
  const router = useRouter();
  const hasError = status === 'failure';

  /**
   * Handles click on the card to navigate to project details
   */
  const handleClick = useCallback(() => {
    router.push(`/projects/github/ps/${project.id}`);
  }, [project.id, router]);

  /**
   * Handles click on the settings menu item
   */
  const handleSettingsClick = useCallback(
    (e: React.MouseEvent) => {
      e.preventDefault();
      e.stopPropagation();
      // Navigate to the settings tab - using the correct path structure
      router.push(`/projects/github/ps/${project.id}/set`);
    },
    [project.id, router]
  );

  /**
   * Handles click on the delete menu item - navigates to settings with delete intent
   */
  const handleDeleteClick = useCallback(
    (e: React.MouseEvent) => {
      e.preventDefault();
      e.stopPropagation();
      // Navigate to settings and add a query parameter to trigger delete modal
      router.push(`/projects/github/ps/${project.id}/set?action=delete`);
    },
    [project.id, router]
  );

  // Extract name and domain from project
  const name = project.full_name || project.name || 'git-account-repo-name';
  const domain = project.deployments?.[0]?.applicationDeploymentRecordData?.url || 'No domain';
  const domainFormatted = domain.replace(/^https?:\/\//, '');

  return (
    <Card 
      className={`bg-background border-border text-foreground w-full cursor-pointer hover:border-muted transition-colors ${className || ''}`} 
      onClick={handleClick} 
      {...props}
    >
      <CardHeader className="flex flex-row items-start justify-between space-y-0 pb-2">
        <div className="flex gap-2.5">
          <div className="flex items-center justify-center w-10 h-10 bg-blue-600 rounded-md text-white font-semibold">
            <Avatar className="h-10 w-10">
              <AvatarImage src={project.icon} alt={name} />
              <AvatarFallback>{getInitials(name)}</AvatarFallback>
            </Avatar>
          </div>

          <div className="flex flex-col gap-1.5">
            <p className="text-sm font-semibold text-foreground leading-none">
              {name}
            </p>
            <p className="text-sm text-muted-foreground leading-5">
              {domainFormatted}
            </p>
          </div>
        </div>

        <div className="flex items-center gap-2">
          {hasError && <AlertTriangle className="text-destructive h-4 w-4" />}
          <ProjectCardActions
            onSettingsClick={handleSettingsClick}
            onDeleteClick={handleDeleteClick}
          />
        </div>
      </CardHeader>

      <CardContent className="flex flex-col gap-3 pt-2">
        <div className="flex items-center gap-2">
          <ProjectStatusDot status={status} />
          <ProjectDeploymentInfo project={project} />
        </div>
      </CardContent>
    </Card>
  );
};