import { Button } from '@workspace/ui/components/button'
import {
  DropdownMenu,
  DropdownMenuContent,
  DropdownMenuItem,
  DropdownMenuSeparator,
  DropdownMenuTrigger
} from '@workspace/ui/components/dropdown-menu'
import { ExternalLink, MoreHorizontal, Trash } from 'lucide-react'
import type React from 'react'
import type { ComponentPropsWithoutRef } from 'react'

/**
 * Props for the ProjectCardActions component
 *
 * @property {Function} onSettingsClick - Callback function triggered when the settings option is clicked
 * @property {Function} onDeleteClick - Callback function triggered when the delete option is clicked
 */
interface ProjectCardActionsProps extends ComponentPropsWithoutRef<'div'> {
  onSettingsClick: (e: React.MouseEvent) => void
  onDeleteClick: (e: React.MouseEvent) => void
}

/**
 * ProjectCardActions component
 *
 * Displays a dropdown menu with actions that can be performed on a project:
 * - Project settings: Navigates to the project settings page
 * - Delete project: Initiates the project deletion process
 *
 * The component uses a three-dot menu icon that expands to show available actions.
 * Each action has an associated icon for better visual recognition.
 *
 * @example
 * ```tsx
 * <ProjectCardActions
 *   onSettingsClick={handleSettingsClick}
 *   onDeleteClick={handleDeleteClick}
 * />
 * ```
 */
export const ProjectCardActions = ({
  onSettingsClick,
  onDeleteClick,
  ...props
}: ProjectCardActionsProps) => {
  return (
    <div {...props}>
      <DropdownMenu>
        <DropdownMenuTrigger asChild>
          <Button variant="ghost" size="icon" className="h-8 w-8">
            <MoreHorizontal className="h-4 w-4" />
            <span className="sr-only">Open menu</span>
          </Button>
        </DropdownMenuTrigger>
        <DropdownMenuContent align="end" className="w-[200px]">
          <DropdownMenuItem onClick={onSettingsClick}>
            <ExternalLink className="mr-2 h-4 w-4" />
            <span>Project settings</span>
          </DropdownMenuItem>
          <DropdownMenuSeparator />
          <DropdownMenuItem
            onClick={onDeleteClick}
            className="text-destructive"
          >
            <Trash className="mr-2 h-4 w-4" />
            <span>Delete project</span>
          </DropdownMenuItem>
        </DropdownMenuContent>
      </DropdownMenu>
    </div>
  )
}
