import { cn } from '@/lib/utils'
import type { ComponentPropsWithoutRef } from 'react'
import type { ProjectStatus } from './ProjectCard'

/**
 * Mapping of status values to their corresponding CSS classes for styling
 *
 * - success: Green color for successful deployments
 * - in-progress: Orange color for deployments in progress
 * - failure: Red color for failed deployments
 * - pending: Gray color for pending deployments
 */
const statusStyles: Record<ProjectStatus, string> = {
  success: 'bg-emerald-500',
  'in-progress': 'bg-orange-400',
  failure: 'bg-destructive',
  pending: 'bg-muted'
}

/**
 * Props for the ProjectStatusDot component
 *
 * @property {ProjectStatus} status - The current status of the project deployment
 */
interface ProjectStatusDotProps extends ComponentPropsWithoutRef<'div'> {
  status: ProjectStatus
}

/**
 * ProjectStatusDot component
 *
 * A visual indicator that displays the current status of a project deployment.
 * The color of the dot changes based on the status:
 * - Green for success
 * - Orange for in-progress
 * - Red for failure
 * - Gray for pending
 *
 * @example
 * ```tsx
 * <ProjectStatusDot status="success" />
 * ```
 */
export const ProjectStatusDot = ({
  status,
  className,
  ...props
}: ProjectStatusDotProps) => {
  return (
    <div
      className={cn('h-2 w-2 rounded-full', statusStyles[status], className)}
      aria-label={`Deployment status: ${status}`}
      {...props}
    />
  )
}
