import { ProjectSearchBarItem } from '@/components/projects/project/ProjectSearchBar/ProjectSearchBarItem'
import { useGQLClient } from '@/context/GQLClientContext'
import * as Dialog from '@radix-ui/react-dialog'
import type { Project } from '@workspace/gql-client'
import { Button } from '@workspace/ui/components/button'
import { Input } from '@workspace/ui/components/input'
import { useCombobox } from 'downshift'
import { Search, X } from 'lucide-react'
import { useRouter } from 'next/router'
import { useCallback, useEffect, useRef, useState } from 'react'
import { useDebounceValue } from 'usehooks-ts'
import { ProjectSearchBarEmpty } from './ProjectSearchBarEmpty'

interface ProjectSearchBarDialogProps extends Dialog.DialogProps {
  open?: boolean
  onClose?: () => void
  onClickItem?: (data: Project) => void
}

export const ProjectSearchBarDialog = ({
  onClose,
  onClickItem,
  ...props
}: ProjectSearchBarDialogProps) => {
  const [items, setItems] = useState<Project[]>([])
  const [selectedItem, setSelectedItem] = useState<Project | null>(null)
  const client = useGQLClient()
  const router = useRouter()
  const inputRef = useRef<HTMLInputElement>(null)

  const {
    getInputProps,
    getItemProps,
    getMenuProps,
    inputValue,
    setInputValue
  } = useCombobox({
    items,
    itemToString(item) {
      return item ? item.name : ''
    },
    selectedItem,
    onSelectedItemChange: ({ selectedItem: newSelectedItem }) => {
      if (newSelectedItem) {
        setSelectedItem(newSelectedItem)
        onClickItem?.(newSelectedItem)
        router.push(
          `/${newSelectedItem.organization.slug}/projects/${newSelectedItem.id}`
        )
      }
    }
  })

  const [debouncedInputValue, _] = useDebounceValue<string>(inputValue, 300)

  const fetchProjects = useCallback(
    async (inputValue: string) => {
      const { searchProjects } = await client.searchProjects(inputValue)
      setItems(searchProjects)
    },
    [client]
  )

  useEffect(() => {
    if (debouncedInputValue) {
      fetchProjects(debouncedInputValue)
    }
  }, [fetchProjects, debouncedInputValue])

  const handleClose = () => {
    setInputValue('')
    setItems([])
    onClose?.()
  }

  return (
    <Dialog.Root {...props}>
      <Dialog.Portal>
        <Dialog.Overlay className="bg-base-bg md:hidden fixed inset-0 overflow-y-auto" />
        <Dialog.Content>
          <div className="fixed inset-0 top-0 flex flex-col h-full">
            <div className="py-2.5 px-4 flex items-center justify-between border-b border-border-separator/[0.06]">
              <div className="relative flex-1">
                <Search className="left-2 top-1/2 text-muted-foreground absolute w-4 h-4 -translate-y-1/2" />
                <Input
                  {...getInputProps(
                    { ref: inputRef },
                    { suppressRefError: true }
                  )}
                  className="pl-8"
                  placeholder="Search"
                  autoFocus
                  type="text"
                />
              </div>
              <Button
                variant="ghost"
                size="icon"
                onClick={handleClose}
                type="button"
              >
                <X className="w-4 h-4" />
              </Button>
            </div>
            <div
              className="flex flex-col gap-1 px-2 py-2"
              {...getMenuProps(
                {},
                {
                  suppressRefError: true
                }
              )}
            >
              {items.length > 0 ? (
                <>
                  <div className="px-2 py-2">
                    <p className="text-elements-mid-em text-xs font-medium">
                      Suggestions
                    </p>
                  </div>
                  {items.map((item, index) => (
                    <ProjectSearchBarItem
                      key={item.id}
                      item={item}
                      {...getItemProps({ item, index })}
                    />
                  ))}
                </>
              ) : (
                inputValue && <ProjectSearchBarEmpty />
              )}
            </div>
          </div>
        </Dialog.Content>
      </Dialog.Portal>
    </Dialog.Root>
  )
}
