import type { Deployment, Domain } from '@/types'
import { relativeTimeMs } from '@/utils/time'
import { GitBranch, MoreVertical } from 'lucide-react'
import {
  DropdownMenu,
  DropdownMenuContent,
  DropdownMenuItem,
  DropdownMenuTrigger,
} from '@workspace/ui/components/dropdown-menu'

interface DeploymentDetailsCardProps {
  deployment: Deployment
  currentDeployment: Deployment
  project: {
    id: string
    prodBranch: string
  }
  prodBranchDomains: Domain[]
}

export function DeploymentDetailsCard({
  deployment,
  currentDeployment,
  project
}: DeploymentDetailsCardProps) {
  const isCurrent = deployment.id === currentDeployment.id
  const isProdBranch = deployment.branch === project.prodBranch
  
  // Get status display styling
  const getStatusStyles = () => {
    // Assuming statuses are: 'BUILDING', 'READY', 'ERROR', 'DELETING' based on your screenshots
    const style = {
      bgColor: '',
      textColor: '',
      label: ''
    };

    switch (deployment.status) {
      case 'BUILDING':
        style.bgColor = 'bg-blue-900/20';
        style.textColor = 'text-blue-400';
        style.label = 'Building';
        break;
      case 'READY':
        style.bgColor = 'bg-green-900/20';
        style.textColor = 'text-green-400';
        style.label = 'Ready';
        break;
      case 'ERROR':
        style.bgColor = 'bg-red-900/20';
        style.textColor = 'text-red-400';
        style.label = 'Error';
        break;
      case 'DELETING':
        style.bgColor = 'bg-gray-800/40';
        style.textColor = 'text-gray-400';
        style.label = 'Deleting';
        break;
      default:
        style.bgColor = 'bg-gray-800/40';
        style.textColor = 'text-gray-400';
        style.label = deployment.status || 'Unknown';
    }

    return style;
  };

  const statusStyle = getStatusStyles();

  return (
    <div className="p-6 border-b border-gray-800 last:border-b-0">
      <div className="flex items-center justify-between">
        <div className="flex items-center gap-4">
          <div className={`px-2 py-1 text-xs rounded-md ${statusStyle.bgColor} ${statusStyle.textColor}`}>
            {statusStyle.label}
          </div>
          <div>Deployer LRN</div>
          <div className="text-gray-400">{deployment.applicationDeploymentRecordData.url.replace(/^https?:\/\//, '')}</div>
        </div>
        
        <DropdownMenu>
          <DropdownMenuTrigger asChild>
            <button className="flex items-center justify-center h-8 w-8 rounded-md hover:bg-gray-800 transition-colors">
              <MoreVertical className="w-4 h-4" />
            </button>
          </DropdownMenuTrigger>
          <DropdownMenuContent align="end">
            <DropdownMenuItem>
              Visit
            </DropdownMenuItem>
            <DropdownMenuItem>
              Change to production
            </DropdownMenuItem>
            <DropdownMenuItem>
              Redeploy to production
            </DropdownMenuItem>
            <DropdownMenuItem>
              Rollback to this version
            </DropdownMenuItem>
            <DropdownMenuItem className="text-red-500">
              Delete deployment
            </DropdownMenuItem>
          </DropdownMenuContent>
        </DropdownMenu>
      </div>
      
      <div className="mt-4 flex flex-col space-y-2">
        <div className="flex items-center gap-2">
          <GitBranch className="w-4 h-4 text-gray-400" />
          <span className="text-sm">{deployment.branch}</span>
          {isCurrent && (
            <span className="text-xs text-gray-400 bg-gray-800 px-2 py-0.5 rounded">
              Current
            </span>
          )}
          {isProdBranch && (
            <span className="text-xs text-gray-400 bg-gray-800 px-2 py-0.5 rounded">
              Production
            </span>
          )}
        </div>
        <div className="flex items-center gap-2 text-sm text-gray-400">
          <span>{deployment.commit || '76d8fb83'}</span>
          <span>deploy: source cargo</span>
          <span className="ml-auto">{relativeTimeMs(deployment.createdAt)}</span>
        </div>
      </div>
    </div>
  )
}