// src/components/templates/TemplateCard.tsx
'use client'

import { ArrowRight, Clock, CheckCircle2 } from 'lucide-react'
import { Button } from '@workspace/ui/components/button'
import { Badge } from '@workspace/ui/components/badge'
import { cn } from '@workspace/ui/lib/utils'
import { TemplateIcon, type TemplateIconType } from './TemplateIcon'
import type { TemplateDetail } from '@/constants/templates'

interface TemplateCardProps {
  template: TemplateDetail
  isSelected?: boolean
  onSelect?: (template: TemplateDetail) => void
  disabled?: boolean
  className?: string
}

export function TemplateCard({ 
  template, 
  isSelected = false, 
  onSelect, 
  disabled = false,
  className 
}: TemplateCardProps) {
  const handleClick = () => {
    if (disabled || template.isComingSoon) return
    onSelect?.(template)
  }

  return (
    <div
      className={cn(
        'group relative flex items-start gap-4 p-4 rounded-lg border transition-all cursor-pointer',
        'hover:border-primary/50 hover:bg-accent/50',
        isSelected && 'border-primary bg-primary/5',
        (disabled || template.isComingSoon) && 'opacity-50 cursor-not-allowed',
        className
      )}
      onClick={handleClick}
    >
      {/* Icon */}
      <div className={cn(
        'flex items-center justify-center w-12 h-12 rounded-lg border',
        'bg-background shadow-sm',
        isSelected && 'border-primary bg-primary/10'
      )}>
        <TemplateIcon 
          type={template.icon as TemplateIconType} 
          size={24}
          className={isSelected ? 'text-primary' : 'text-muted-foreground'}
        />
      </div>

      {/* Content */}
      <div className="flex-1 min-w-0">
        <div className="flex items-start justify-between mb-2">
          <h3 className="font-medium text-sm leading-tight">
            {template.name}
          </h3>
          
          {/* Selection indicator */}
          {isSelected && (
            <CheckCircle2 className="h-5 w-5 text-primary flex-shrink-0" />
          )}
        </div>

        <p className="text-xs text-muted-foreground leading-relaxed mb-3">
          {template.description}
        </p>

        {/* Tags */}
        {template.tags && template.tags.length > 0 && (
          <div className="flex flex-wrap gap-1 mb-3">
            {template.tags.map((tag) => (
              <Badge 
                key={tag} 
                variant="secondary" 
                className="text-xs px-2 py-0.5"
              >
                {tag}
              </Badge>
            ))}
          </div>
        )}

        {/* Footer */}
        <div className="flex items-center justify-between text-xs">
          <span className="text-muted-foreground font-mono">
            {template.repoFullName}
          </span>
          
          {template.isComingSoon ? (
            <Badge variant="outline" className="text-xs">
              <Clock className="w-3 h-3 mr-1" />
              Coming Soon
            </Badge>
          ) : (
            <Button
              variant="ghost"
              size="sm"
              className={cn(
                'h-auto p-1 opacity-0 group-hover:opacity-100 transition-opacity',
                isSelected && 'opacity-100'
              )}
            >
              <ArrowRight className="h-3 w-3" />
            </Button>
          )}
        </div>
      </div>
    </div>
  )
}