// src/components/templates/TemplateSelection.tsx
'use client'

import { useState } from 'react'
import { Card, CardContent, CardHeader, CardTitle } from '@workspace/ui/components/card'
import { Alert, AlertDescription } from '@workspace/ui/components/alert'
import { Badge } from '@workspace/ui/components/badge'
import { Info, Sparkles } from 'lucide-react'
import { TemplateCard } from './TemplateCard'
import { AVAILABLE_TEMPLATES, type TemplateDetail } from '@/constants/templates'

interface TemplateSelectionProps {
  selectedTemplate?: TemplateDetail
  onTemplateSelect: (template: TemplateDetail) => void
  disabled?: boolean
  className?: string
}

export function TemplateSelection({ 
  selectedTemplate, 
  onTemplateSelect, 
  disabled = false,
  className 
}: TemplateSelectionProps) {
  const [hoveredTemplate, setHoveredTemplate] = useState<string | null>(null)

  const availableTemplates = AVAILABLE_TEMPLATES.filter(t => !t.isComingSoon)
  const comingSoonTemplates = AVAILABLE_TEMPLATES.filter(t => t.isComingSoon)

  return (
    <Card className={className}>
      <CardHeader>
        <CardTitle className="flex items-center gap-2 text-base">
          <Sparkles className="h-5 w-5 text-primary" />
          Choose a Template
        </CardTitle>
      </CardHeader>
      <CardContent className="space-y-4">
        <Alert>
          <Info className="h-4 w-4" />
          <AlertDescription className="text-sm">
            Templates create a new repository in your GitHub account with pre-configured code.
            You can customize it after deployment.
          </AlertDescription>
        </Alert>

        {/* Available Templates */}
        <div className="space-y-3">
          <div className="flex items-center justify-between">
            <h3 className="text-sm font-medium">Available Templates</h3>
            <Badge variant="secondary" className="text-xs">
              {availableTemplates.length} templates
            </Badge>
          </div>
          
          <div className="grid gap-3">
            {availableTemplates.map((template) => (
              <TemplateCard
                key={template.id}
                template={template}
                isSelected={selectedTemplate?.id === template.id}
                onSelect={onTemplateSelect}
                disabled={disabled}
              />
            ))}
          </div>
        </div>

        {/* Coming Soon Templates */}
        {comingSoonTemplates.length > 0 && (
          <div className="space-y-3">
            <div className="flex items-center justify-between">
              <h3 className="text-sm font-medium text-muted-foreground">Coming Soon</h3>
              <Badge variant="outline" className="text-xs">
                {comingSoonTemplates.length} more
              </Badge>
            </div>
            
            <div className="grid gap-3">
              {comingSoonTemplates.map((template) => (
                <TemplateCard
                  key={template.id}
                  template={template}
                  disabled={true}
                />
              ))}
            </div>
          </div>
        )}

        {/* Selection Summary */}
        {selectedTemplate && (
          <div className="p-3 bg-primary/5 border border-primary/20 rounded-md">
            <div className="flex items-center gap-2 mb-1">
              <div className="w-2 h-2 bg-primary rounded-full"></div>
              <span className="text-sm font-medium">Selected Template</span>
            </div>
            <p className="text-sm text-muted-foreground">
              <strong>{selectedTemplate.name}</strong> will be forked to your GitHub account
            </p>
            <p className="text-xs text-muted-foreground mt-1 font-mono">
              Source: {selectedTemplate.repoFullName}
            </p>
          </div>
        )}
      </CardContent>
    </Card>
  )
}