import type { GQLClient } from '@workspace/gql-client'
import { type ReactNode, createContext, useContext } from 'react'

/**
 * @const GQLClientContext
 * @description Creates a context for managing the GQLClient instance.
 */
const GQLClientContext = createContext({} as GQLClient)

/**
 * @component GQLClientProvider
 * @description Provides the GQLClientContext to its children.
 * @param {Object} props - The component props
 * @param {ReactNode} props.children - The children to render.
 * @param {GQLClient} props.client - The GQLClient instance.
 */
export const GQLClientProvider = ({
  client,
  children
}: {
  children: ReactNode
  client: GQLClient
}) => (
  <GQLClientContext.Provider value={client}>
    {children}
  </GQLClientContext.Provider>
)

/**
 * @function useGQLClient
 * @description A hook that provides access to the GQLClientContext.
 * @returns {GQLClient} The GQLClient instance.
 * @throws {Error} If used outside of a GQLClientProvider.
 */
export const useGQLClient = () => {
  const client = useContext(GQLClientContext)
  if (!client) {
    throw new Error('useGQLClient must be used within a GQLClientProvider')
  }
  return client
}
