// src/context/OctokitContext.tsx
import { Octokit, RequestError } from 'octokit'

import { useDebounceCallback } from 'usehooks-ts'

import { useParams, useRouter } from 'next/navigation'
import {
  type ReactNode,
  createContext,
  useCallback,
  useContext,
  useEffect,
  useMemo,
  useState
} from 'react'
import { toast } from 'sonner'
import { useGQLClient } from './GQLClientContext'

const UNAUTHORIZED_ERROR_CODE = 401

/**
 * @interface ContextValue
 * @description Defines the structure of the OctokitContext value.
 * @property {Octokit} octokit - The Octokit instance.
 * @property {boolean} isAuth - Indicates if the user is authenticated with GitHub.
 * @property {function} updateAuth - Function to update the authentication status.
 */
interface ContextValue {
  octokit: Octokit
  isAuth: boolean
  updateAuth: () => void
}

/**
 * @const OctokitContext
 * @description Creates a context for managing Octokit and authentication state.
 */
const OctokitContext = createContext<ContextValue>({
  octokit: new Octokit(),
  isAuth: false,
  updateAuth: () => {}
})

/**
 * @component OctokitProvider
 * @description Provides the OctokitContext to its children.
 * @param {Object} props - Component props
 * @param {ReactNode} props.children - The children to render.
 * @param {Function} [props.navigate] - Optional navigation function. If not provided, useNavigate will be used.
 */
export const OctokitProvider = ({
  children
}: {
  children: ReactNode
  navigate?: (to: string) => void
}) => {
  const [authToken, setAuthToken] = useState<string | null>(null)
  const [isAuth, setIsAuth] = useState(false)
  const router = useRouter()
  const { orgSlug } = useParams()
  const gqlClient = useGQLClient()

  /**
   * @function fetchUser
   * @description Fetches the user's GitHub token from the GQLClient.
   */
  const fetchUser = useCallback(async () => {
    try {
      console.log('🔍 Fetching user data from GraphQL...')
      
      // Check if gqlClient exists and has the getUser method
      if (!gqlClient || typeof gqlClient.getUser !== 'function') {
        console.error('❌ GQL client not available or getUser method missing')
        setAuthToken(null)
        return
      }

      const { user } = await gqlClient.getUser()
      console.log('📊 User data received:', user)
      
      if (user && user.gitHubToken) {
        console.log('✅ GitHub token found in user data')
        setAuthToken(user.gitHubToken)
      } else {
        console.log('❌ No GitHub token found in user data')
        setAuthToken(null)
      }
    } catch (error) {
      console.error('❌ Error fetching user:', error)
      setAuthToken(null)
    }
  }, [gqlClient])

  /**
   * @function updateAuth
   * @description Updates the authentication status by fetching the user.
   */
  const updateAuth = useCallback(() => {
    fetchUser()
  }, [fetchUser])

  const octokit = useMemo(() => {
    if (!authToken) {
      setIsAuth(false)
      console.log('🐙 Creating Octokit without auth token')
      return new Octokit()
    }

    console.log('🐙 Creating Octokit with auth token')
    setIsAuth(true)
    return new Octokit({ auth: authToken })
  }, [authToken])

  // Only fetch user when GQL client is available
  useEffect(() => {
    if (gqlClient) {
      console.log('🔄 GQL client available, fetching user...')
      fetchUser()
    } else {
      console.log('⏳ Waiting for GQL client...')
    }
  }, [gqlClient, fetchUser])

  const debouncedUnauthorizedGithubHandler = useDebounceCallback(
    useCallback(
      (error: RequestError) => {
        toast.error(`GitHub authentication error: ${error.message}`)

        router.push(`/${orgSlug}/projects/create`)
      },
      [orgSlug, router]
    ),
    500
  )

  useEffect(() => {
    // TODO: Handle React component error
    const interceptor = async (error: RequestError | Error) => {
      if (
        error instanceof RequestError &&
        error.status === UNAUTHORIZED_ERROR_CODE
      ) {
        if (gqlClient && typeof gqlClient.unauthenticateGithub === 'function') {
          await gqlClient.unauthenticateGithub()
        }
        await fetchUser()

        debouncedUnauthorizedGithubHandler(error)
      }

      throw error
    }

    octokit.hook.error('request', interceptor)

    return () => {
      // Remove the interceptor when the component unmounts
      octokit.hook.remove('request', interceptor)
    }
  }, [octokit, gqlClient, debouncedUnauthorizedGithubHandler, fetchUser])

  return (
    <OctokitContext.Provider value={{ octokit, updateAuth, isAuth }}>
      {children}
    </OctokitContext.Provider>
  )
}

/**
 * @function useOctokit
 * @description A hook that provides access to the OctokitContext.
 * @returns {object} An object containing the Octokit instance, updateAuth function, and isAuth status.
 */
export const useOctokit = () => {
  const context = useContext(OctokitContext)

  if (!context) {
    throw new Error('useOctokit must be used within an OctokitProvider')
  }

  return context
}