// src/hooks/useAuthStatus.tsx
'use client'

import { useAuth, useUser } from '@clerk/nextjs'
import { useWallet } from '@/context/WalletContext' // Use the full provider!
import { useBackend } from '@/context/BackendContext'
import { useGQLClient } from '@/context'
import { useState, useEffect, useCallback } from 'react'

/**
 * @interface AuthStatus
 * @description Comprehensive authentication status across all systems
 */
export interface AuthStatus {
  // Individual auth systems
  clerk: {
    isSignedIn: boolean
    isLoaded: boolean
    hasGithubConnected: boolean
    user: any
  }
  wallet: {
    isConnected: boolean        // SIWE authenticated + backend session  
    hasAddress: boolean         // Just has wallet address
    wallet: any
  }
  backend: {
    isConnected: boolean
    hasGithubAuth: boolean
    isLoading: boolean
  }
  
  // Computed status
  isFullyAuthenticated: boolean
  isReady: boolean
  
  // What's missing (for UI feedback)
  missing: {
    clerkSignIn: boolean
    clerkGithub: boolean
    walletConnection: boolean
    backendConnection: boolean
    githubBackendSync: boolean
  }
  
  // Progress (for UI indicators)
  progress: {
    completed: number
    total: number
    percentage: number
  }
}

/**
 * @interface AuthActions
 * @description Available authentication actions
 */
export interface AuthActions {
  // Wallet actions
  connectWallet: () => Promise<void>
  
  // Combined actions
  refreshAllStatus: () => Promise<void>
  checkGithubBackendAuth: () => Promise<boolean>
}

/**
 * @hook useAuthStatus
 * @description Provides comprehensive authentication status and actions
 * @returns Combined auth status and actions
 */
export function useAuthStatus(): AuthStatus & AuthActions {
  // Clerk authentication
  const { isSignedIn, isLoaded: isClerkLoaded } = useAuth()
  const { user, isLoaded: isUserLoaded } = useUser()
  
  // Wallet authentication
  const { 
    isConnected: isWalletSessionActive,  // SIWE authenticated
    hasWalletAddress, 
    wallet, 
    connect: connectWallet 
  } = useWallet()
  
  // Backend authentication
  const {
    isBackendConnected,
    isLoading: isBackendLoading,
    refreshStatus: refreshBackendStatus
  } = useBackend()
  
  // GraphQL client for checking GitHub backend auth
  const gqlClient = useGQLClient()
  
  // GitHub backend auth state
  const [isGithubBackendAuth, setIsGithubBackendAuth] = useState(false)
  const [isCheckingGithubAuth, setIsCheckingGithubAuth] = useState(false)
  
  // Check GitHub backend auth via GraphQL
  const checkGithubBackendAuth = useCallback(async (): Promise<boolean> => {
    if (!isBackendConnected) return false
    
    try {
      setIsCheckingGithubAuth(true)
      const userData = await gqlClient.getUser()
      const hasGitHubToken = !!userData.user.gitHubToken
      setIsGithubBackendAuth(hasGitHubToken)
      return hasGitHubToken
    } catch (error) {
      console.error('Error checking GitHub backend auth:', error)
      setIsGithubBackendAuth(false)
      return false
    } finally {
      setIsCheckingGithubAuth(false)
    }
  }, [isBackendConnected, gqlClient])
  
  // Check GitHub auth when backend connection changes
  useEffect(() => {
    if (isBackendConnected) {
      checkGithubBackendAuth()
    } else {
      setIsGithubBackendAuth(false)
    }
  }, [isBackendConnected, checkGithubBackendAuth])
  
  // Check backend connection when wallet session is active (SIWE completed)
  useEffect(() => {
    if (isWalletSessionActive) {
      // Wait a moment for wallet session to be established, then check backend
      const timer = setTimeout(() => {
        refreshBackendStatus()
      }, 1000)
      return () => clearTimeout(timer)
    }
  }, [isWalletSessionActive, refreshBackendStatus])
  
  // Check if GitHub is connected in Clerk
  const hasGithubInClerk = user?.externalAccounts?.find(
    account => account.provider === 'github' || account.verification?.strategy === 'oauth_github'
  ) !== undefined
  
  // Calculate what's missing
  const missing = {
    clerkSignIn: !isSignedIn,
    clerkGithub: isSignedIn && !hasGithubInClerk,
    walletConnection: !hasWalletAddress, // Just need wallet address for this step
    backendConnection: hasWalletAddress && !isWalletSessionActive, // Need SIWE auth for backend
    githubBackendSync: isBackendConnected && !isGithubBackendAuth
  }
  
  // Calculate progress
  const authSteps = [
    isSignedIn,                    // Clerk sign in
    hasGithubInClerk,             // GitHub connected to Clerk
    hasWalletAddress,             // Wallet address obtained
    isWalletSessionActive,        // SIWE authentication completed  
    isGithubBackendAuth           // GitHub synced to backend
  ]
  
  const completedSteps = authSteps.filter(Boolean).length
  const totalSteps = authSteps.length
  const progressPercentage = Math.round((completedSteps / totalSteps) * 100)
  
  // Determine if fully authenticated
  const isFullyAuthenticated = authSteps.every(Boolean)
  
  // Determine if ready (all auth systems loaded)
  const isReady = isClerkLoaded && isUserLoaded && !isBackendLoading && !isCheckingGithubAuth
  
  // Combined refresh action
  const refreshAllStatus = async () => {
    await refreshBackendStatus()
    await checkGithubBackendAuth()
  }
  
  return {
    // Individual systems
    clerk: {
      isSignedIn,
      isLoaded: isClerkLoaded && isUserLoaded,
      hasGithubConnected: hasGithubInClerk,
      user
    },
    wallet: {
      isConnected: isWalletSessionActive,
      hasAddress: hasWalletAddress,
      wallet
    },
    backend: {
      isConnected: isBackendConnected,
      hasGithubAuth: isGithubBackendAuth,
      isLoading: isBackendLoading || isCheckingGithubAuth
    },
    
    // Computed status
    isFullyAuthenticated,
    isReady,
    
    // Missing items
    missing,
    
    // Progress
    progress: {
      completed: completedSteps,
      total: totalSteps,
      percentage: progressPercentage
    },
    
    // Actions
    connectWallet,
    refreshAllStatus,
    checkGithubBackendAuth
  }
}