"use client";

import { useState, useEffect } from "react";
import { useAuth, useUser } from "@clerk/nextjs";
import { Octokit } from "@octokit/rest";

// Define the return type of the hook
interface UseRepoDataReturn {
  repoData: any;
  isLoading: boolean;
  error: string | null;
}

/**
 * A hook to fetch repository data from GitHub
 * 
 * @param repoId - The GitHub repository ID to fetch, or empty string to fetch all repos
 * @returns Object containing repository data, loading state, and any errors
 */
export function useRepoData(repoId: string): UseRepoDataReturn {
  const [repoData, setRepoData] = useState<any>(null);
  const [isLoading, setIsLoading] = useState(true);
  const [error, setError] = useState<string | null>(null);
  const [octokit, setOctokit] = useState<Octokit | null>(null);
  
  // Get auth data from Clerk
  const { isLoaded: isAuthLoaded } = useAuth();
  const { isLoaded: isUserLoaded, user } = useUser();

  // Initialize Octokit with the appropriate token
  useEffect(() => {
    async function initializeOctokit() {
      let token = null;
      
      // Try to get GitHub token from Clerk
      if (user) {
        try {
          // Check if user has connected GitHub account
          const githubAccount = user.externalAccounts.find(
            account => account.provider === 'oauth_github'
          );
          
          if (githubAccount) {
            // Try to get GitHub OAuth token from Clerk
            try {
              token = await user.getToken({ template: 'github' });
              console.log('Using GitHub token from Clerk');
            } catch (err) {
              console.error('Error getting GitHub token from Clerk:', err);
            }
          }
        } catch (err) {
          console.error('Error accessing Clerk user data:', err);
        }
      }
      
      // Fallback to token from environment variable
      if (!token && typeof process !== 'undefined') {
        token = process.env.NEXT_PUBLIC_GITHUB_FALLBACK_TOKEN || '';
        if (token) {
          console.warn('Using fallback GitHub token. This should only be used for development.');
        }
      }
      
      // Create Octokit instance with whatever token we found
      if (token) {
        setOctokit(new Octokit({ auth: token }));
      } else {
        setError("No GitHub token available");
        setIsLoading(false);
      }
    }
    
    if (isAuthLoaded && isUserLoaded) {
      initializeOctokit();
    }
  }, [isAuthLoaded, isUserLoaded, user]);

  // Fetch repo data when Octokit is available
  useEffect(() => {
    let isMounted = true;
    
    async function fetchRepoData() {
      if (!octokit) {
        return;
      }
      
      try {
        // Fetch repos from GitHub
        const { data: repos } = await octokit.repos.listForAuthenticatedUser();
        
        // If no repoId is provided, return all repos
        if (!repoId) {
          if (isMounted) {
            setRepoData(repos);
            setError(null);
            setIsLoading(false);
          }
          return;
        }
        
        // Find the specific repo by ID if repoId is provided
        const repo = repos.find(repo => repo.id.toString() === repoId);
        
        if (!repo) {
          if (isMounted) {
            setError("Repository not found");
            setRepoData(null);
            setIsLoading(false);
          }
        } else {
          if (isMounted) {
            setRepoData(repo);
            setError(null);
            setIsLoading(false);
          }
        }
      } catch (err) {
        console.error('Error fetching GitHub repo:', err);
        if (isMounted) {
          setError('Failed to fetch repository data');
          setRepoData(null);
          setIsLoading(false);
        }
      }
    }
    
    if (octokit) {
      fetchRepoData();
    }
    
    return () => {
      isMounted = false;
    };
  }, [repoId, octokit]);

  return { repoData, isLoading, error };
}