// src/hooks/useTemplateDeployment.tsx
'use client'

import { useState } from 'react'
import { useGQLClient } from '@/context'
import { useWallet } from '@/context/WalletContext'
import { useUser } from '@clerk/nextjs'
import { toast } from 'sonner'
import type { TemplateDetail } from '@/constants/templates'

export interface TemplateDeploymentConfig {
  template: TemplateDetail
  projectName: string
  organizationSlug: string
  environmentVariables?: Array<{
    key: string
    value: string
    environments: string[]
  }>
  deployerLrn?: string
}

export interface TemplateDeploymentResult {
  projectId: string
  repositoryUrl: string
  deploymentUrl?: string
  deploymentId?: string
}

export function useTemplateDeployment() {
  const [isDeploying, setIsDeploying] = useState(false)
  const [deploymentResult, setDeploymentResult] = useState<TemplateDeploymentResult | null>(null)
  const [error, setError] = useState<string | null>(null)
  
  const gqlClient = useGQLClient()
  const { wallet } = useWallet()
  const { user } = useUser()

  const deployTemplate = async (config: TemplateDeploymentConfig): Promise<TemplateDeploymentResult> => {
    setIsDeploying(true)
    setError(null)
    setDeploymentResult(null)
    
    try {
      console.log('🚀 Starting template deployment:', config)
      
      // Validate required data
      if (!wallet?.address) {
        throw new Error('Wallet not connected')
      }
      
      if (!user) {
        throw new Error('User not authenticated')
      }
      
      // Get GitHub username from Clerk external accounts
      const githubAccount = user.externalAccounts?.find(account => account.provider === 'github')
      const githubUsername = githubAccount?.username
      
      if (!githubUsername) {
        throw new Error('GitHub account not connected')
      }
      
      console.log('🔍 GitHub user info:', {
        githubUsername,
        githubAccount: githubAccount?.username,
        userExternalAccounts: user.externalAccounts?.length
      })
      
      // Parse template repository (format: "owner/repo")
      const [templateOwner, templateRepo] = config.template.repoFullName.split('/')
      if (!templateOwner || !templateRepo) {
        throw new Error('Invalid template repository format')
      }
      
      // Debug logging
      console.log('🔍 Template parsing details:', {
        originalTemplate: config.template.repoFullName,
        parsedOwner: templateOwner,
        parsedRepo: templateRepo,
        templateId: config.template.id,
        templateName: config.template.name
      })
      
      const requestData = {
        templateOwner,
        templateRepo,
        owner: githubUsername, // Use the authenticated GitHub username
        name: config.projectName,
        isPrivate: false,
        paymentAddress: wallet.address,
        txHash: "0x0000000000000000000000000000000000000000000000000000000000000000" // Placeholder - will be updated if payment is required
      }
      
      console.log('🔍 Request data being sent to backend:', requestData)
      console.log('🔍 Organization slug:', config.organizationSlug)
      console.log('🔍 Deployer LRN:', config.deployerLrn)
      console.log('🔍 Environment variables:', config.environmentVariables)
      
      toast.info('Creating repository from template...')
      
      // Use the backend's addProjectFromTemplate method
      const projectResult = await gqlClient.addProjectFromTemplate(
        config.organizationSlug,
        requestData,
        config.deployerLrn, // deployer LRN for direct deployment
        undefined, // auctionParams - not used for LRN deployments
        config.environmentVariables || []
      )
      
      console.log('🔍 Backend response:', projectResult)
      
      if (!projectResult.addProjectFromTemplate?.id) {
        throw new Error('Failed to create project from template')
      }
      
      toast.success('Repository created from template!')
      
      // Wait for deployment to start
      await new Promise(resolve => setTimeout(resolve, 3000))
      
      // Get project with deployment info
      const projectData = await gqlClient.getProject(projectResult.addProjectFromTemplate.id)
      
      console.log('🔍 Project data after creation:', projectData)
      
      if (!projectData.project) {
        throw new Error('Project not found after creation')
      }
      
      const deployment = projectData.project.deployments?.[0]
      
      const result: TemplateDeploymentResult = {
        projectId: projectResult.addProjectFromTemplate.id,
        repositoryUrl: `https://github.com/${projectData.project.repository}`,
        deploymentUrl: deployment?.applicationDeploymentRecordData?.url,
        deploymentId: deployment?.id
      }
      
      console.log('🔍 Final deployment result:', result)
      
      setDeploymentResult(result)
      toast.success('Template deployed successfully!')
      
      return result
      
    } catch (error) {
      console.error('❌ Template deployment failed:', error)
      const errorMessage = error instanceof Error ? error.message : 'Unknown error'
      setError(errorMessage)
      toast.error(`Template deployment failed: ${errorMessage}`)
      throw error
    } finally {
      setIsDeploying(false)
    }
  }

  const getTemplateInfo = async (templateRepo: string) => {
    try {
      // This would fetch template information if needed
      // For now, we can just return the repo name
      return {
        name: templateRepo,
        description: `Template from ${templateRepo}`
      }
    } catch (error) {
      console.error('Error fetching template info:', error)
      throw error
    }
  }

  const reset = () => {
    setDeploymentResult(null)
    setError(null)
    setIsDeploying(false)
  }

  return {
    deployTemplate,
    getTemplateInfo,
    isDeploying,
    deploymentResult,
    error,
    reset
  }
}