import { Slot } from '@radix-ui/react-slot'
import { type VariantProps, cva } from 'class-variance-authority'
import type * as React from 'react'

import { cn } from '@workspace/ui/lib/utils'

const buttonVariants = cva(
  "inline-flex items-center justify-center gap-2 whitespace-nowrap rounded-md text-sm font-medium transition-[color,box-shadow] disabled:pointer-events-none disabled:opacity-50 [&_svg]:pointer-events-none [&_svg:not([class*='size-'])]:size-4 shrink-0 [&_svg]:shrink-0 outline-none focus-visible:border-ring focus-visible:ring-ring/50 focus-visible:ring-[3px] aria-invalid:ring-destructive/20 dark:aria-invalid:ring-destructive/40 aria-invalid:border-destructive",
  {
    variants: {
      variant: {
        default:
          'bg-primary text-primary-foreground shadow-xs hover:bg-primary/90',
        destructive:
          'bg-destructive text-white shadow-xs hover:bg-destructive/90 focus-visible:ring-destructive/20 dark:focus-visible:ring-destructive/40',
        outline:
          'border border-input bg-background shadow-xs hover:bg-accent hover:text-accent-foreground',
        secondary:
          'bg-secondary text-secondary-foreground shadow-xs hover:bg-secondary/80',
        ghost: 'hover:bg-accent hover:text-accent-foreground',
        link: 'text-primary underline-offset-4 hover:underline'
      },
      size: {
        default: 'h-9 px-4 py-2 has-[>svg]:px-3',
        sm: 'h-8 rounded-md gap-1.5 px-3 has-[>svg]:px-2.5',
        lg: 'h-10 rounded-md px-6 has-[>svg]:px-4',
        icon: 'size-9'
      }
    },
    defaultVariants: {
      variant: 'default',
      size: 'default'
    }
  }
)

function Button({
  className,
  variant,
  size,
  asChild = false,
  ...props
}: React.ComponentProps<'button'> &
  VariantProps<typeof buttonVariants> & { asChild?: boolean }) {
  const Comp = asChild ? Slot : 'button'

  return (
    <Comp
      data-slot="button"
      className={cn(buttonVariants({ variant, size, className }))}
      {...props}
    />
  )
}

/**
 * Props for the IconButton component
 * @interface IconButtonProps
 * @extends {React.ComponentProps<'button'>}
 * @extends {VariantProps<typeof buttonVariants>}
 */
interface IconButtonProps
  extends Omit<React.ComponentProps<'button'>, 'children'>,
    VariantProps<typeof buttonVariants> {
  /** Icon component to display on the left side */
  leftIcon?: React.ReactNode
  /** Icon component to display on the right side */
  rightIcon?: React.ReactNode
  /** Optional text content */
  children?: React.ReactNode
  /** Whether to render as a child component */
  asChild?: boolean
}

/**
 * A button component that can display icons on either or both sides
 * @param props - The component props
 * @returns A button with optional icons and text
 */
function IconButton({
  className,
  variant,
  size = 'default',
  leftIcon,
  rightIcon,
  children,
  asChild = false,
  ...props
}: IconButtonProps) {
  const Comp = asChild ? Slot : 'button'
  const hasContent = Boolean(children)
  const finalSize = !hasContent && (leftIcon || rightIcon) ? 'icon' : size

  return (
    <Comp
      data-slot="button"
      className={cn(buttonVariants({ variant, size: finalSize, className }))}
      {...props}
    >
      {leftIcon}
      {children}
      {rightIcon}
    </Comp>
  )
}

export { Button, IconButton, buttonVariants }
